@@ -48,12 +48,20 @@ if sys.version_info[0] == 2: # pragma: no 3.x cover
     def _as_bytes(value):
         return value
 
+    def _as_string(value):
+        return value
+
 else: # pragma: no 2.x cover
     def _as_bytes(value):
         if isinstance(value, bytes):
             return value
         return value.encode('ascii')
 
+    def _as_string(value):
+        if isinstance(value, bytes):
+            return value.decode('ascii')
+        return value
+
 class _BridgeSupportParser (object):
     """
     Parser for the bridge support file format.
@@ -94,6 +102,12 @@ class _BridgeSupportParser (object):
     def typestr2typestr(self, typestr):
         typestr = _as_bytes(typestr)
 
+        # As of macOS 10.13 metadata files may contain
+        # typestring that end with property specific data;
+        # first remove that junk.
+        if b',' in typestr:
+            typestr = typestr.split(b',', 1)[0]
+
         result = []
         for item in objc.splitSignature(typestr):
             if item == objc._C_BOOL:
@@ -108,8 +122,9 @@ class _BridgeSupportParser (object):
 
                 name, fields = objc.splitStructSignature(objc._C_STRUCT_B + _as_bytes(item[1:-1]) + objc._C_STRUCT_E)
                 result.append(start)
-                result.append(_as_bytes(name))
-                result.append(b'=')
+                if name is not None:
+                    result.append(_as_bytes(name))
+                    result.append(b'=')
                 for nm, tp in fields:
                     if nm is not None:
                         result.append(b'"')
@@ -121,7 +136,7 @@ class _BridgeSupportParser (object):
 
 
             elif item.startswith(objc._C_ARY_B):
-                m = re.match(b'^.(\d*)(.*).$', item)
+                m = re.match(br'^.(\d*)(.*).$', item)
                 result.append(objc._C_ARY_B)
                 result.append(m.group(1))
                 result.append(self.typestr2typestr(m.group(2)))
@@ -130,8 +145,7 @@ class _BridgeSupportParser (object):
             else:
                 result.append(item)
 
-        result = b''.join(result)
-        return result
+        return b''.join(result)
 
 
     if sys.maxsize > 2**32:
@@ -409,6 +423,9 @@ class _BridgeSupportParser (object):
             else:
                 value = float(value)
 
+        elif 'inf' in value:
+            value = float(value)
+
         else:
             value = int(value, 10)
 
@@ -607,7 +624,7 @@ def parseBridgeSupport(xmldata, globals, frameworkName, dylib_path=None, inlineT
 
         for name, typestr, magic in prs.constants:
             try:
-                value = objc._loadConstant(name, typestr, magic)
+                value = objc._loadConstant(name, _as_string(typestr), magic)
             except AttributeError:
                 continue
 
@@ -774,6 +791,10 @@ def _structConvenience(structname, structencoding):
             return objc.ivar(type=structencoding)
         else:
             return objc.ivar(name=name, type=structencoding)
+    makevar.__name__ = structname
+    makevar.__doc__ = "Create *ivar* for type encoding %r" % (structencoding,)
+    if hasattr(objc.ivar, '__qualname__'):
+        makevar.__qualname__ = objc.ivar.__qualname__ + "." + structname
     _ivar_dict[structname] = classmethod(makevar)
 
 
