@@ -12,8 +12,16 @@ import struct
 
 from objc import lookUpClass, getClassList, nosuchclass_error, loadBundle
 import objc
+import warnings
 ModuleType = type(sys)
 
+_name_re = re.compile('^[A-Za-z_][A-Za-z_0-9]*$')
+
+def _check_deprecated(name, deprecation_version):
+    if objc.options.deprecation_warnings and objc.options.deprecation_warnings >= deprecation_version:
+        warnings.warn("%r is deprecated in macOS %d.%d"%(name, deprecation_version / 100, deprecation_version % 100),
+                objc.ApiDeprecationWarning, stacklevel=2)
+
 def _loadBundle(frameworkName, frameworkIdentifier, frameworkPath):
     if frameworkIdentifier is None:
         bundle = loadBundle(
@@ -78,8 +86,10 @@ class ObjCLazyModule (ModuleType):
             self.__bundle = None
 
         pfx = name + '.'
-        for nm in sys.modules:
-            if nm.startswith(pfx):
+        for nm in list(sys.modules.keys()):
+            # See issue #95: there can be objects that aren't strings in
+            # sys.modules.
+            if hasattr(nm, 'startswith') and nm.startswith(pfx):
                 rest = nm[len(pfx):]
                 if '.' in rest: continue
                 if sys.modules[nm] is not None:
@@ -93,10 +103,13 @@ class ObjCLazyModule (ModuleType):
         self.__dict__.update(metadict.get('misc', {}))
         self.__parents = parents
         self.__varmap = metadict.get('constants')
+        self.__varmap_deprecated = metadict.get('deprecated_constants', {})
         self.__varmap_dct = metadict.get('constants_dict', {})
         self.__enummap = metadict.get('enums')
+        self.__enum_deprecated = metadict.get('deprecated_enums', {})
         self.__funcmap = metadict.get('functions')
         self.__aliases = metadict.get('aliases')
+        self.__aliases_deprecated = metadict.get('deprecated_aliases', {})
         self.__inlinelist = inline_list
 
         # informal protocols are not exposed, but added here
@@ -119,7 +132,7 @@ class ObjCLazyModule (ModuleType):
             self.__dict__[name] = value
             return value
 
-        # First try parent module, as we had done
+        # First try parent module, as if we had done
         # 'from parents import *'
         for p in self.__parents:
             try:
@@ -133,6 +146,11 @@ class ObjCLazyModule (ModuleType):
                     del self.__dict__['__all__']
                 return value
 
+        if not _name_re.match(name):
+            # Name is not a valid identifier and cannot
+            # match.
+            raise AttributeError(name)
+
         # Check if the name is a constant from
         # the metadata files
         try:
@@ -173,9 +191,15 @@ class ObjCLazyModule (ModuleType):
                     self.__dict__[nm] = dct[nm]
 
             for nm, tp in self.__varmap_dct.items():
-                if tp.startswith('='):
+                if tp.startswith('=='):
                     try:
-                        self.__dict__[nm] = objc._loadConstant(nm, tp[1:], True)
+                        self.__dict__[nm] = objc._loadConstant(nm, tp[2:], 2)
+                    except AttributeError:
+                        raise
+                        pass
+                elif tp.startswith('='):
+                    try:
+                        self.__dict__[nm] = objc._loadConstant(nm, tp[1:], 1)
                     except AttributeError:
                         pass
 
@@ -200,7 +224,10 @@ class ObjCLazyModule (ModuleType):
 
             for nm, tp in specials:
                 try:
-                    self.__dict__[nm] = objc._loadConstant(nm, tp, True)
+                    if tp.startswith('='):
+                        self.__dict__[nm] = objc._loadConstant(nm, tp[1:], 2)
+                    else:
+                        self.__dict__[nm] = objc._loadConstant(nm, tp, 1)
                 except AttributeError:
                     pass
 
@@ -285,12 +312,18 @@ class ObjCLazyModule (ModuleType):
         if self.__varmap_dct:
             if name in self.__varmap_dct:
                 tp = self.__varmap_dct.pop(name)
-                if tp.startswith('='):
+                if tp.startswith('=='):
+                    tp = tp[2:]
+                    magic = 2
+                elif tp.startswith('='):
                     tp = tp[1:]
-                    magic = True
+                    magic = 1
                 else:
-                    magic = False
+                    magic = 0
                 result = objc._loadConstant(name, tp, magic)
+                if name in self.__varmap_deprecated:
+                    _check_deprecated(name, self.__varmap_deprecated[name])
+
                 return result
 
         if self.__varmap:
@@ -303,18 +336,29 @@ class ObjCLazyModule (ModuleType):
                     tp = tp[1:]
 
                 d = {}
-                if tp.startswith('='):
+                if tp.startswith('=='):
+                    magic = 2
+                    tp = tp[2:]
+                elif tp.startswith('='):
                     tp = tp[1:]
-                    magic = True
+                    magic = 1
                 else:
-                    magic = False
+                    magic = 0
 
-                return objc._loadConstant(name, tp, magic)
+                result =  objc._loadConstant(name, tp, magic)
+
+                if name in self.__varmap_deprecated:
+                    _check_deprecated(name, self.__varmap_deprecated[name])
+
+                return result
 
         if self.__enummap:
             m = re.search(r"\$%s@([^$]*)\$"%(name,), self.__enummap)
             if m is not None:
-                return self.__prs_enum(m.group(1))
+                result =  self.__prs_enum(m.group(1))
+                if name in self.__enum_deprecated:
+                    _check_deprecated(name, self.__enum_deprecated[name])
+                return result
 
         if self.__funcmap:
             if name in self.__funcmap:
@@ -352,13 +396,25 @@ class ObjCLazyModule (ModuleType):
             if name in self.__aliases:
                 alias = self.__aliases.pop(name)
                 if alias == 'ULONG_MAX':
-                    return (sys.maxsize * 2) + 1
+                    result = (sys.maxsize * 2) + 1
                 elif alias == 'LONG_MAX':
-                    return sys.maxsize
+                    result = sys.maxsize
                 elif alias == 'LONG_MIN':
-                    return -sys.maxsize-1
+                    result = -sys.maxsize-1
+                elif alias == 'DBL_MAX':
+                    result = sys.float_info.max
+                elif alias == 'DBL_MIN':
+                    result = sys.float_info.min
+                elif alias == 'FLT_MAX':
+                    result = objc._FLT_MAX
+                elif alias == 'FLT_MIN':
+                    result = objc._FLT_MIN
+                else:
+                    result = getattr(self, alias)
 
-                return getattr(self, alias)
+                if name in self.__aliases_deprecated:
+                    _check_deprecated(name, self.__aliases_deprecated[name])
+                return result
 
         raise AttributeError(name)
 
