@@ -7,7 +7,7 @@
 {
     OC_PythonData* res;
 
-    res = [[OC_PythonData alloc] initWithPythonObject:v];
+    res = [[self alloc] initWithPythonObject:v];
     [res autorelease];
     return res;
 }
@@ -47,9 +47,18 @@
     return YES;
 }
 
++ (BOOL)supportsSecureCoding {
+    return NO;
+}
+
 -(oneway void)release
 {
     /* See comment in OC_PythonUnicode */
+    if (unlikely(!Py_IsInitialized())) {
+        [super release];
+        return;
+    }
+
     PyObjC_BEGIN_WITH_GIL
         [super release];
 
@@ -60,6 +69,10 @@
 
 -(void)dealloc
 {
+    if (unlikely(!Py_IsInitialized())) {
+        [super dealloc];
+        return;
+    }
     PyObjC_BEGIN_WITH_GIL
         PyObjC_UnregisterObjCProxy(value, self);
         Py_XDECREF(value);
@@ -118,7 +131,7 @@
 #if PY_VERSION_HEX >= 0x02060000
     } else if (PyByteArray_CheckExact(value)) {
         return [NSMutableData class];
-#endif /* PY_VERSION_HEX >= 0x02060000  */
+#endif /* PY_VERSION_HEX >= 0x02060000 */
 
     } else {
         return [OC_PythonData class];
@@ -149,7 +162,7 @@
 
             }
             [super encodeWithCoder:coder];
-#endif /* PY_VERSION_HEX >= 0x02060000  */
+#endif /* PY_VERSION_HEX >= 0x02060000 */
 
         } else {
             if ([coder allowsKeyedCoding]) {
@@ -314,8 +327,8 @@
 }
 
 /* Ensure that we can be unarchived as a generic string by pure ObjC
- *  * code.
- *   */
+ * code.
+ */
 +(NSArray*)classFallbacksForKeyedArchiver
 {
     return [NSArray arrayWithObject:@"NSData"];
