@@ -7,7 +7,7 @@
 {
     OC_PythonDate* res;
 
-    res = [[OC_PythonDate alloc] initWithPythonObject:v];
+    res = [[self alloc] initWithPythonObject:v];
     return [res autorelease];
 }
 
@@ -39,9 +39,18 @@
     return YES;
 }
 
++ (BOOL)supportsSecureCoding {
+    return NO;
+}
+
 -(oneway void)release
 {
     /* See comment in OC_PythonUnicode */
+    if (unlikely(!Py_IsInitialized())) {
+        [super release];
+        return;
+    }
+
     PyObjC_BEGIN_WITH_GIL
         [super release];
 
@@ -50,9 +59,14 @@
 
 -(void)dealloc
 {
-    [oc_value  release];
+    [oc_value release];
     oc_value = nil;
 
+    if (unlikely(!Py_IsInitialized())) {
+        [super dealloc];
+        return;
+    }
+
     PyObjC_BEGIN_WITH_GIL
         PyObjC_UnregisterObjCProxy(value, self);
         Py_XDECREF(value);
@@ -135,8 +149,10 @@
                 PyObjC_GIL_FORWARD_EXC();
             }
 
-
+#pragma clang diagnostic push
+#pragma clang diagnostic ignored "-Wdeprecated-declarations"
             oc_value = [NSDate dateWithString: PyObjC_PythonToId(v)];
+#pragma clang diagnostic pop
             [oc_value retain];
             Py_DECREF(v);
 
@@ -169,7 +185,10 @@
                     /* Raise ObjC exception */
                 }
 
+#pragma clang diagnostic push
+#pragma clang diagnostic ignored "-Wdeprecated-declarations"
                 oc_value = [NSDate dateWithString: PyObjC_PythonToId(v)];
+#pragma clang diagnostic pop
                 [oc_value retain];
                 Py_DECREF(v);
             }
