@@ -84,7 +84,7 @@
 
 +(OC_PythonDictionary*)dictionaryWithPythonObject:(PyObject*)v
 {
-    OC_PythonDictionary* res = [[OC_PythonDictionary alloc] initWithPythonObject:v];
+    OC_PythonDictionary* res = [[self alloc] initWithPythonObject:v];
     [res autorelease];
     return res;
 }
@@ -102,9 +102,18 @@
     return YES;
 }
 
++ (BOOL)supportsSecureCoding {
+    return NO;
+}
+
 -(oneway void)release
 {
     /* See comment in OC_PythonUnicode */
+    if (unlikely(!Py_IsInitialized())) {
+        [super release];
+        return;
+    }
+
     PyObjC_BEGIN_WITH_GIL
         [super release];
 
@@ -113,6 +122,11 @@
 
 -(void)dealloc
 {
+    if (unlikely(!Py_IsInitialized())) {
+        [super dealloc];
+        return;
+    }
+
     PyObjC_BEGIN_WITH_GIL
         PyObjC_UnregisterObjCProxy(value, self);
         Py_CLEAR(value);
@@ -339,7 +353,7 @@
     NSUInteger i;
 
     PyObjC_BEGIN_WITH_GIL
-        for  (i = 0; i < count; i++) {
+        for (i = 0; i < count; i++) {
             PyObject* k;
             PyObject* v;
             int r;
@@ -571,4 +585,4 @@
     }
 }
 
-@end  // interface OC_PythonDictionary
+@end /* interface OC_PythonDictionary */
