@@ -10,11 +10,19 @@
         if (PyErr_Occurred()) {
             PyErr_Clear();
         } else if (lv >= 1ULL<<63) {
+            /* Workaround for round-trip problems... */
+#ifdef __clang__
+#pragma clang diagnostic push
+#pragma clang diagnostic ignored "-Wincompatible-pointer-types"
+#endif
             return [[NSNumber alloc] initWithUnsignedLongLong:lv];
+#ifdef __clang__
+#pragma clang diagnostic pop
+#endif
         }
     }
 
-    res = [[OC_PythonNumber alloc] initWithPythonObject:v];
+    res = [[self alloc] initWithPythonObject:v];
     [res autorelease];
     return res;
 }
@@ -45,9 +53,18 @@
     return YES;
 }
 
++ (BOOL)supportsSecureCoding {
+    return NO;
+}
+
 -(oneway void)release
 {
     /* See comment in OC_PythonUnicode */
+    if (unlikely(!Py_IsInitialized())) {
+        [super release];
+        return;
+    }
+
     PyObjC_BEGIN_WITH_GIL
         [super release];
 
@@ -56,9 +73,14 @@
 
 -(void)dealloc
 {
+    if (unlikely(!Py_IsInitialized())) {
+        [super dealloc];
+        return;
+    }
+
     PyObjC_BEGIN_WITH_GIL
         PyObjC_UnregisterObjCProxy(value, self);
-        Py_XDECREF(value);
+        Py_CLEAR(value);
 
     PyObjC_END_WITH_GIL
 
@@ -90,6 +112,9 @@
                     PyObjC_GIL_RETURN(@encode(unsigned long long));
                 }
                 PyErr_Clear();
+
+		/* Wrap on overflow */
+	        PyObjC_GIL_RETURN(@encode(long long));
             }
         }
     PyObjC_END_WITH_GIL
@@ -113,7 +138,6 @@
 -(void)getValue:(void*)buffer forType:(const char*)type
 {
     int r;
-    NSLog(@"getValue %p forType %s", buffer, type);
     PyObjC_BEGIN_WITH_GIL
         r = depythonify_c_value(type, value, buffer);
         if (r == -1) {
@@ -219,15 +243,15 @@
     PyObjC_BEGIN_WITH_GIL
 #if PY_MAJOR_VERSION == 2
         if (PyInt_Check(value)) {
-            result =  PyInt_AsLong(value);
+            result = PyInt_AsLong(value);
             PyObjC_GIL_RETURN(result);
         } else
 #endif
         if (PyFloat_Check(value)) {
-            result =  (long long)PyFloat_AsDouble(value);
+            result = (long long)PyFloat_AsDouble(value);
             PyObjC_GIL_RETURN(result);
         } else if (PyLong_Check(value)) {
-            result =  PyLong_AsUnsignedLongLongMask(value);
+            result = PyLong_AsUnsignedLongLongMask(value);
             PyObjC_GIL_RETURN(result);
         }
     PyObjC_END_WITH_GIL
@@ -243,12 +267,12 @@
 
     PyObjC_BEGIN_WITH_GIL
         if (PyLong_Check(value)) {
-            result =  PyLong_AsUnsignedLongLongMask(value);
+            result = PyLong_AsUnsignedLongLongMask(value);
             PyObjC_GIL_RETURN(result);
 
 #if PY_MAJOR_VERSION == 2
         } else if (PyInt_Check(value)) {
-            result =  (unsigned long long)PyInt_AsLong(value);
+            result = (unsigned long long)PyInt_AsLong(value);
             PyObjC_GIL_RETURN(result);
 #endif
 
@@ -265,7 +289,7 @@
                 long long t = (long long)temp;
                 result = (unsigned long long)t;
             } else {
-                result =  (unsigned long long)temp;
+                result = (unsigned long long)temp;
             }
             PyObjC_GIL_RETURN(result);
         }
@@ -286,6 +310,7 @@
     PyObject* repr;
     NSObject* result = nil;
 
+
     PyObjC_BEGIN_WITH_GIL
         repr = PyObject_Repr(value);
         if (repr == NULL) {
@@ -356,7 +381,11 @@
 
     if (use_super) {
         [super encodeWithCoder:coder];
+
     } else {
+        /* XXX: Should check if coder requiresSecureCoding, and bail out in
+         * that case.
+         */
         PyObjC_encodeWithCoder(value, coder);
     }
 }
@@ -429,7 +458,7 @@
      *
      * In all other cases use Python's comparison semantics.
      */
-    if ([aNumber isKindOfClass:[NSNumber class]] && ![aNumber isMemberOfClass: [OC_PythonNumber class]]) {
+    if ([aNumber isKindOfClass:[NSNumber class]] && ![aNumber isKindOfClass: [OC_PythonNumber class]]) {
         int use_super = 0;
 
         PyObjC_BEGIN_WITH_GIL
@@ -437,7 +466,6 @@
                 PY_LONG_LONG r;
                 r = PyLong_AsLongLong(value);
                 if (r == -1 && PyErr_Occurred()) {
-                    PyErr_Print();
                     PyErr_Clear();
                 } else {
                     use_super = 1;
@@ -515,7 +543,7 @@ COMPARE_METHOD(isLessThanOrEqualTo, Py_LE)
     return [self isEqualTo:aNumber];
 }
 
-
+#if 0
 -(NSObject*)replacementObjectForArchiver:(NSArchiver*)archiver
 {
     (void)archiver;
@@ -539,6 +567,7 @@ COMPARE_METHOD(isLessThanOrEqualTo, Py_LE)
     (void)archiver;
     return (NSObject*)self;
 }
+#endif
 
 -(Class)classForArchiver
 {
@@ -561,13 +590,13 @@ COMPARE_METHOD(isLessThanOrEqualTo, Py_LE)
             (void)PyLong_AsLongLong(value);
             if (PyErr_Occurred()) {
                 PyErr_Clear();
-                PyObjC_GIL_RETURN([OC_PythonNumber class]);
+                PyObjC_GIL_RETURN([self class]);
             } else {
                 PyObjC_GIL_RETURN([NSNumber class]);
             }
 
         } else {
-            PyObjC_GIL_RETURN([OC_PythonNumber class]);
+            PyObjC_GIL_RETURN([self class]);
         }
     PyObjC_END_WITH_GIL
 }
@@ -577,26 +606,11 @@ COMPARE_METHOD(isLessThanOrEqualTo, Py_LE)
     return [self classForArchiver];
 }
 
-+(Class)classForUnarchiver
-{
-    return [OC_PythonNumber class];
-}
-
-+(Class)classForKeyedUnarchiver
-{
-    return [OC_PythonNumber class];
-}
-
 -(Class)classForCoder
 {
     return [self classForArchiver];
 }
 
--(Class)classForPortCoder
-{
-    return [self classForArchiver];
-}
-
 -(id)copy
 {
     return [self copyWithZone:0];
