@@ -182,7 +182,13 @@ static char* keywords[] = { "bundle", "module_globals", "variableInfo", "skip_un
             }
 
         } else {
-            PyObject* py_val = pythonify_c_value(signature, value);
+            PyObject* py_val;
+
+            if (*signature == _C_CHARPTR) {
+                py_val = pythonify_c_value(signature, &value);
+            } else {
+                py_val = pythonify_c_value(signature, value);
+            }
             if (py_val == NULL) {
                 Py_DECREF(seq);
                 return NULL;
@@ -232,7 +238,7 @@ static char* keywords[] = { "bundle", "module_globals", "functionInfo", "skip_un
 
         PyObjC_ENDHANDLER
 
-        if  (cfBundle == NULL && PyErr_Occurred()) {
+        if (cfBundle == NULL && PyErr_Occurred()) {
             return NULL;
         }
 
@@ -348,7 +354,7 @@ static char* keywords[] = { "bundle", "module_globals", "functionInfo", "skip_un
 
 typedef void(*function)(void);
 struct functionlist {
-    char*     name;
+    char* name;
     function func;
 };
 
