@@ -5,6 +5,6 @@ extern Class PyObjCClass_BuildClass(Class, PyObject*, char*,
         PyObject*, PyObject*, PyObject*, PyObject*);
 extern int PyObjCClass_UnbuildClass(Class);
 extern int PyObjCClass_FinishClass(Class);
-extern void PyObjC_RemoveInternalTypeCodes(char*);
+extern int PyObjC_RemoveInternalTypeCodes(char*);
 
 #endif /* OBJC_CLASS_BUILDER */
