@@ -250,10 +250,14 @@ determine_if_shortcut(PyObjCMethodSignature* methinfo)
     Py_ssize_t byref_in_count = 0, byref_out_count = 0, plain_count = 0, argbuf_len = 0;
     BOOL variadic_args = NO;
 
+    if (methinfo == 0) {
+        PyErr_SetString(PyObjCExc_InternalError, "methinfo not set");
+        return -1;
+    }
     methinfo->shortcut_signature = NO;
     methinfo->shortcut_argbuf_size = 0;
 
-    return 0;
+    /* return 0; */
 
     if (methinfo == NULL || methinfo->variadic) {
         return 0;
@@ -282,6 +286,7 @@ determine_if_shortcut(PyObjCMethodSignature* methinfo)
     }
 
     methinfo->shortcut_signature = YES;
+    methinfo->shortcut_signature = NO; /* XXX */
     methinfo->shortcut_argbuf_size = (unsigned int)argbuf_len;
     return 0;
 }
@@ -399,6 +404,9 @@ new_methodsignature(const char* signature)
         }
         nargs++;
     }
+    if (cur == NULL && PyErr_Occurred()) {
+        return NULL;
+    }
 
     retval = PyObject_NewVar(PyObjCMethodSignature,
             &PyObjCMethodSignature_Type, nargs /*+1*/);
@@ -415,6 +423,7 @@ new_methodsignature(const char* signature)
     Py_SIZE(retval) = nargs;
     retval->suggestion = NULL;
     retval->variadic = NO;
+    retval->deprecated = 0;
     retval->free_result = NO;
     retval->shortcut_signature = NO;
     retval->shortcut_argbuf_size = 0;
@@ -667,7 +676,7 @@ setup_descr(struct _PyObjC_ArgDescr* descr, PyObject* meta, BOOL is_native)
         if (d) {
             if (descr == NULL || descr->tmpl) return -2;
 
-            /* Make up a dummy signature, will be overridden bij
+            /* Make up a dummy signature, will be overridden by
              * the metadata.
              */
             char buffer[64];
@@ -886,6 +895,9 @@ setup_descr(struct _PyObjC_ArgDescr* descr, PyObject* meta, BOOL is_native)
 
         if (descr == NULL || descr->tmpl) return -2;
 
+        descr->modifier = typeModifier;
+
+
         if (PyUnicode_Check(d)) {
             bytes = PyUnicode_AsEncodedString(d, NULL, NULL);
             if (bytes == NULL) {
@@ -913,6 +925,7 @@ setup_descr(struct _PyObjC_ArgDescr* descr, PyObject* meta, BOOL is_native)
             /* The new signature is not compatible enough, ignore the
              * override.
              */
+            printf("Types not compatible: %s   %s\n", descr->type, type);
             type = descr->type;
         }
 
@@ -965,7 +978,7 @@ setup_descr(struct _PyObjC_ArgDescr* descr, PyObject* meta, BOOL is_native)
                 return -1;
             }
 
-            tp[0]  = typeModifier;
+            tp[0] = typeModifier;
             strcpy(tp+1, withoutModifiers);
 
             if (descr->typeOverride) {
@@ -1081,6 +1094,21 @@ process_metadata_dict(PyObjCMethodSignature* methinfo, PyObject* metadata, BOOL
             Py_INCREF(v);
         }
 
+        v = PyDict_GetItemString(metadata, "deprecated");
+        if (v) {
+            if (PyLong_Check(v)) {
+                methinfo->deprecated = (int)PyLong_AsLong(v);
+                if (PyErr_Occurred()) {
+                    return -1;
+                }
+            }
+#if PY_MAJOR_VERSION == 2
+            else if (PyInt_Check(v)) {
+                methinfo->deprecated = (int)PyInt_AsLong(v);
+            }
+#endif
+        }
+
         methinfo->null_terminated_array = NO;
         v = PyDict_GetItemString(metadata, "c_array_delimited_by_null");
         if (v && PyObject_IsTrue(v)) {
@@ -1189,6 +1217,7 @@ compiled_metadata(PyObject* metadata)
     Py_SIZE(result) = max_idx;
     result->suggestion = NULL;
     result->variadic = NO;
+    result->deprecated = 0;
     result->free_result = NO;
     result->shortcut_signature = NO;
     result->shortcut_argbuf_size = 0;
@@ -1370,7 +1399,7 @@ merge_descr(struct _PyObjC_ArgDescr* descr, struct _PyObjC_ArgDescr* meta, BOOL
 
             /* Skip existing modifiers, we're overriding those */
             strcpy(tp+1, withoutModifiers);
-            tp[0]  = meta->modifier;
+            tp[0] = meta->modifier;
             PyObjC_Assert(tp != NULL, NULL);
             descr->typeOverride = YES;
             descr->type = tp;
@@ -1405,6 +1434,7 @@ process_metadata_object(PyObjCMethodSignature* methinfo, PyObjCMethodSignature*
     methinfo->null_terminated_array = metadata->null_terminated_array;
     methinfo->free_result = metadata->free_result;
     methinfo->arrayArg = metadata->arrayArg;
+    methinfo->deprecated = metadata->deprecated;
 
     if (methinfo->rettype->tmpl && metadata->rettype != NULL && metadata->rettype->modifier != '\0' && is_default_descr(metadata->rettype)) {
         const char* withoutModifiers = PyObjCRT_SkipTypeQualifiers(methinfo->rettype->type);
@@ -1476,7 +1506,7 @@ PyObjCMethodSignature* PyObjCMethodSignature_ForSelector(
     }
 
     if (isClassMethod) {
-        const char* nm  = sel_getName(sel);
+        const char* nm = sel_getName(sel);
         if (strncmp(nm, "new", 3) == 0 && ((nm[3] == 0) || isupper(nm[3]))) {
             if (methinfo->rettype->tmpl) {
                 methinfo->rettype = alloc_descr(methinfo->rettype);
@@ -1520,12 +1550,16 @@ argdescr2dict(struct _PyObjC_ArgDescr* descr)
      * method signature.
      */
     if (descr->type != NULL) {
-        end = PyObjCRT_SkipTypeSpec(descr->type) - 1;
+        end = PyObjCRT_SkipTypeSpec(descr->type);
+        if (unlikely(end == NULL)) {
+            goto error;
+        }
+        end--;
         while ((end != descr->type) && isdigit(*end)) {
             end --;
         }
         end ++;
-        v = PyBytes_FromStringAndSize(descr->type,  end - descr->type);
+        v = PyBytes_FromStringAndSize(descr->type, end - descr->type);
         if (v == NULL) goto error;
         r = PyDict_SetItemString(result, "type", v);
         Py_DECREF(v);
@@ -1675,6 +1709,15 @@ PyObjCMethodSignature_AsDict(PyObjCMethodSignature* methinfo)
         if (r == -1) goto error;
     }
 
+    if (methinfo->deprecated != 0) {
+        v = Py_BuildValue("i", &methinfo->deprecated);
+        if (v == NULL) goto error;
+
+        r = PyDict_SetItemString(result, "deprecated", v);
+        Py_DECREF(v);
+        if (r == -1) goto error;
+    }
+
     if (methinfo->rettype == NULL) {
         v = Py_None; Py_INCREF(Py_None);
     } else {
@@ -1710,3 +1753,9 @@ error:
     Py_XDECREF(result);
     return NULL;
 }
+
+PyObject*
+PyObjC_copyMetadataRegistry(void)
+{
+    return PyObjC_CopyRegistry(registry, (PyObjC_ItemTransform)PyObjCMethodSignature_AsDict);
+}
