@@ -24,12 +24,80 @@
 
 #include <dlfcn.h>
 
+static int PyObjC_Initialized = 0;
+
 PyObject* PyObjCClass_DefaultModule = NULL;
 
 PyObject* PyObjC_TypeStr2CFTypeID = NULL;
 
 static NSAutoreleasePool* global_release_pool = nil;
 
+/* Calculate the current version of macOS in a format that
+ * can be compared with MAC_OS_VERSION_X_... constants
+ */
+
+#if PyObjC_BUILD_RELEASE < 1010
+typedef struct {
+    long versionMajor;
+    long versionMinor;
+    long versionPatch;
+} NSOperatingSystemVersion;
+#endif
+
+static NSOperatingSystemVersion gSystemVersion = { 0, 0, 0 };
+
+static long
+calc_current_version(void)
+{
+#if PyObjC_BUILD_RELEASE >= 1010
+    if ([NSProcessInfo instancesRespondToSelector:@selector(operatingSystemVersion)]) {
+        NSAutoreleasePool* pool;
+
+        pool = [[NSAutoreleasePool alloc] init];
+        gSystemVersion = [[NSProcessInfo processInfo] operatingSystemVersion];
+	[pool release];
+
+
+    } else
+#endif
+    {
+	/* Code path for macOS 10.9 or earlier. Don't use Gestalt because that's deprecated. */
+        NSAutoreleasePool* pool;
+        NSDictionary* plist;
+        NSArray* parts;
+
+        pool = [[NSAutoreleasePool alloc] init];
+
+        plist = [NSDictionary dictionaryWithContentsOfFile:@"/System/Library/CoreServices/SystemVersion.plist"];
+        if (!plist) {
+            NSLog(@"Cannot determine system version");
+            return 0;
+        }
+
+        parts = [[plist valueForKey:@"ProductVersion"] componentsSeparatedByString:@"."];
+
+        if (!parts || [parts count] < 2) {
+            NSLog(@"Cannot determine system version");
+            return 0;
+        }
+
+        gSystemVersion.majorVersion = [[parts objectAtIndex:0] intValue];
+        gSystemVersion.minorVersion = [[parts objectAtIndex:1] intValue];
+
+        if (parts.count >= 3) {
+            gSystemVersion.patchVersion = [[parts objectAtIndex:2] intValue];
+        }
+
+        [pool release];
+    }
+
+    if (gSystemVersion.majorVersion >= 10 || gSystemVersion.minorVersion >= 10) {
+        return gSystemVersion.majorVersion * 10000 + gSystemVersion.minorVersion * 100 + gSystemVersion.patchVersion;
+    } else {
+        return gSystemVersion.majorVersion * 100 + gSystemVersion.minorVersion;
+    }
+}
+
 @interface OC_NSAutoreleasePoolCollector: NSObject
   /*
    * This class is used to automaticly reset the
@@ -142,6 +210,52 @@ PyObject* PyObjCStrBridgeWarning = NULL;
 
 #endif /* PY_VERSION_MAJOR == 2 */
 
+
+PyDoc_STRVAR(macos_available_doc,
+  "macos_available(major, minor, patch=0)\n"
+  CLINIC_SEP
+  "\n"
+  "Return true if the current macOS release is "
+  "at least the provided version");
+
+static PyObject*
+macos_available(PyObject* self __attribute__((__unused__)),
+    PyObject* args, PyObject* kwds)
+{
+static char* keywords[] = { "major", "minor", "patch", NULL };
+    long major;
+    long minor;
+    long patch = 0;
+
+    if (!PyArg_ParseTupleAndKeywords(args, kwds, "ll|l",
+            keywords, &major, &minor, &patch)) {
+        return NULL;
+    }
+
+    if (major > gSystemVersion.majorVersion) {
+        Py_RETURN_FALSE;
+    } else if (major == gSystemVersion.majorVersion) {
+        if (minor > gSystemVersion.minorVersion) {
+             Py_RETURN_FALSE;
+        } else if (minor == gSystemVersion.minorVersion) {
+             if (patch > gSystemVersion.patchVersion) {
+                 Py_RETURN_FALSE;
+             } else {
+                 Py_RETURN_TRUE;
+             }
+        } else {
+             Py_RETURN_TRUE;
+        }
+
+    } else {
+        Py_RETURN_TRUE;
+    }
+}
+
+
+
+
+
 PyDoc_STRVAR(lookUpClass_doc,
   "lookUpClass(class_name)\n"
   CLINIC_SEP
@@ -184,7 +298,7 @@ static PyObject*
 classAddMethods(PyObject* self __attribute__((__unused__)),
     PyObject* args, PyObject* keywds)
 {
-    static     char* kwlist[] = { "targetClass", "methodsArray", NULL };
+    static char* kwlist[] = { "targetClass", "methodsArray", NULL };
     PyObject* classObject = NULL;
     PyObject* methodsArray = NULL;
 
@@ -220,7 +334,7 @@ PyDoc_STRVAR(remove_autorelease_pool_doc,
   "removeAutoreleasePool()\n"
   CLINIC_SEP
   "\n"
-  "This removes the global NSAutoreleasePool.  You should do this\n"
+  "This removes the global NSAutoreleasePool. You should do this\n"
   "at the end of a plugin's initialization script.\n");
 
 static PyObject*
@@ -256,7 +370,7 @@ static PyObject*
 recycle_autorelease_pool(PyObject* self __attribute__((__unused__)),
     PyObject* args, PyObject* kwds)
 {
-    static    char* keywords[] = { NULL };
+    static char* keywords[] = { NULL };
 
     if (!PyArg_ParseTupleAndKeywords(args, kwds, "", keywords)) {
         return NULL;
@@ -302,7 +416,7 @@ static PyObject*
 set_class_extender(PyObject* self __attribute__((__unused__)),
     PyObject* args, PyObject* kwds)
 {
-static     char* keywords[] = { "callback", NULL };
+static char* keywords[] = { "callback", NULL };
     PyObject* callback;
 
     if (!PyArg_ParseTupleAndKeywords(args, kwds, "O:setClassExtender",
@@ -345,7 +459,7 @@ PyDoc_STRVAR(allocateBuffer_doc,
 static PyObject*
 allocateBuffer(PyObject* self __attribute__((__unused__)), PyObject* args, PyObject* kwds)
 {
-    static    char* keywords[] = { "length", 0 };
+    static char* keywords[] = { "length", 0 };
     Py_ssize_t length;
 
     if (!PyArg_ParseTupleAndKeywords(args, kwds, "n",
@@ -408,8 +522,8 @@ PyDoc_STRVAR(loadBundle_doc,
 static PyObject*
 loadBundle(PyObject* self __attribute__((__unused__)), PyObject* args, PyObject* kwds)
 {
-static  char* keywords[] = { "module_name", "module_globals", "bundle_path", "bundle_identifier", "scan_classes", NULL };
-static    Py_ssize_t    curClassCount = -1;
+static char* keywords[] = { "module_name", "module_globals", "bundle_path", "bundle_identifier", "scan_classes", NULL };
+static Py_ssize_t curClassCount = -1;
     NSBundle* bundle = nil;
     id bundle_identifier = nil;
     id bundle_path = nil;
@@ -482,7 +596,7 @@ static    Py_ssize_t    curClassCount = -1;
     curClassCount = len = PyTuple_GET_SIZE(class_list);
     for (i = 0; i < len; i++) {
         PyObject* item;
-        const char*  nm;
+        const char* nm;
 
         item = PyTuple_GET_ITEM(class_list, i);
         if (item == NULL) {
@@ -517,7 +631,7 @@ PyDoc_STRVAR(objc_splitSignature_doc,
 static PyObject*
 objc_splitSignature(PyObject* self __attribute__((__unused__)), PyObject* args, PyObject* kwds)
 {
-static  char* keywords[] = { "signature", NULL };
+static char* keywords[] = { "signature", NULL };
     const char* signature;
     const char* end;
     PyObject* result;
@@ -579,7 +693,7 @@ PyDoc_STRVAR(objc_splitStructSignature_doc,
 static PyObject*
 objc_splitStructSignature(PyObject* self __attribute__((__unused__)), PyObject* args, PyObject* kwds)
 {
-static  char* keywords[] = { "signature", NULL };
+static char* keywords[] = { "signature", NULL };
     const char* signature;
     const char* end;
     PyObject* structname;
@@ -795,7 +909,7 @@ PyDoc_STRVAR(protocolNamed_doc,
 static PyObject*
 protocolNamed(PyObject* self __attribute__((__unused__)), PyObject* args, PyObject* kwds)
 {
-static     char* keywords[] = { "name", NULL };
+static char* keywords[] = { "name", NULL };
     char* name;
     Protocol* p;
 
@@ -885,6 +999,22 @@ static char* keywords[] = { "name", "typestr", "doc", NULL };
     return PyObjCCreateOpaquePointerType(name, typestr, docstr);
 }
 
+PyDoc_STRVAR(copyMetadataRegistry_doc,
+    "_copyMetadataRegistry()\n"
+    CLINIC_SEP
+    "\n"
+    "Return a copy of the metdata registry.");
+static PyObject*
+copyMetadataRegistry(PyObject* self __attribute__((__unused__)), PyObject* args, PyObject* kwds)
+{
+static char* keywords[] = { NULL };
+
+    if (!PyArg_ParseTupleAndKeywords(args, kwds, "", keywords)) {
+        return NULL;
+    }
+    return PyObjC_copyMetadataRegistry();
+}
+
 PyDoc_STRVAR(registerMetaData_doc,
     "registerMetaDataForSelector(classObject, selector, metadata)\n"
     CLINIC_SEP
@@ -908,6 +1038,7 @@ static char* keywords[] = { "class_", "selector", "metadata", NULL };
         return NULL;
 
     } else {
+        PyObjC_MappingCount ++;
         Py_INCREF(Py_None);
         return Py_None;
     }
@@ -1259,7 +1390,7 @@ _makeClosure(
     PyObject* args,
     PyObject* kwds)
 {
-static  char* keywords[] = { "callable", "closureFor", "argIndex", NULL };
+static char* keywords[] = { "callable", "closureFor", "argIndex", NULL };
     PyObject* callable;
     PyObject* closureFor;
     PyObjCMethodSignature* methinfo;
@@ -1341,6 +1472,36 @@ static  char* keywords[] = { "callable", "closureFor", "argIndex", NULL };
     return Py_BuildValue("NN", retval, PyObjCMethodSignature_AsDict(methinfo->argtype[argIndex]->callable));
 }
 
+PyDoc_STRVAR(_closurePointer_doc,
+  "_closurePointer(closure)\n"
+  CLINIC_SEP
+  "\n"
+  "Returns an integer that corresponds to the numeric value of the C pointer\n"
+  "for the closure."
+ );
+static PyObject*
+_closurePointer(
+    PyObject* self __attribute__((__unused__)),
+    PyObject* args,
+    PyObject* kwds)
+{
+static char* keywords[] = { "closure", NULL };
+    PyObject* closure;
+    void*     pointer;
+
+
+    if (!PyArg_ParseTupleAndKeywords(args, kwds, "O",
+        keywords, &closure)) {
+        return NULL;
+    }
+
+    pointer = PyCapsule_GetPointer(closure, "objc.__imp__");
+    if (pointer == NULL && PyErr_Occurred()) {
+        return NULL;
+    }
+    return PyLong_FromVoidPtr(pointer);
+}
+
 static PyObject*
 ivar_dict(PyObject* self __attribute__((__unused__)))
 {
@@ -1482,7 +1643,7 @@ typestr2typestr(PyObject* args)
     return result;
 }
 
-#if    PyObjC_BUILD_RELEASE >= 1006
+#if PyObjC_BUILD_RELEASE >= 1006
     /* Associated Object support. Functionality is available on OSX 10.6 or later. */
 
 PyDoc_STRVAR(PyObjC_setAssociatedObject_doc,
@@ -1501,7 +1662,7 @@ static char* keywords[] = { "object", "key", "value", "policy", NULL };
     id value;
     long policy = OBJC_ASSOCIATION_RETAIN;
 
-    if (objc_setAssociatedObject == NULL) {
+    if (&objc_setAssociatedObject == NULL) {
         PyErr_SetString(PyObjCExc_Error, "setAssociatedObject not available on this platform");
         return NULL;
     }
@@ -1547,7 +1708,7 @@ static char* keywords[] = { "object", "key", NULL};
     PyObject* key;
     id value;
 
-    if (objc_getAssociatedObject == NULL) {
+    if (&objc_getAssociatedObject == NULL) {
         PyErr_SetString(PyObjCExc_Error, "setAssociatedObject not available on this platform");
         return NULL;
     }
@@ -1588,7 +1749,7 @@ PyObjC_removeAssociatedObjects(PyObject* self __attribute__((__unused__)),
 static char* keywords[] = { "object", NULL};
     id object;
 
-    if (objc_removeAssociatedObjects == NULL) {
+    if (&objc_removeAssociatedObjects == NULL) {
         PyErr_SetString(PyObjCExc_Error, "removeAssociatedObjects not available on this platform");
         return NULL;
     }
@@ -1623,7 +1784,7 @@ PyObjC_LoadConstant(PyObject* self __attribute__((__unused__)),
 static char* keywords[] = { "name", "type", "magic", NULL };
     char* name;
     char* type;
-    int   magic;
+    int magic;
 
     if (!PyArg_ParseTupleAndKeywords(args, kwds, "ssi",
         keywords, &name, &type, &magic)) {
@@ -1640,9 +1801,17 @@ static char* keywords[] = { "name", "type", "magic", NULL };
     PyObject* v;
 
     if (magic) {
-        v = PyObjCCF_NewSpecial(type, buf);
+        if (magic == 2) {
+            v = PyObjCCF_NewSpecial(type, *(void**)buf);
+        } else {
+            v = PyObjCCF_NewSpecial(type, buf);
+        }
     } else {
-        v = pythonify_c_value(type, buf);
+        if (*type == _C_CHARPTR) {
+            v = pythonify_c_value(type, &buf);
+        } else {
+            v = pythonify_c_value(type, buf);
+        }
     }
 
     return v;
@@ -1776,6 +1945,12 @@ static PyMethodDef mod_methods[] = {
         .ml_flags   = METH_VARARGS|METH_KEYWORDS,
         .ml_doc     = objc_splitStructSignature_doc,
     },
+    {
+        .ml_name    = "macos_available",
+        .ml_meth    = (PyCFunction)macos_available,
+        .ml_flags   = METH_VARARGS|METH_KEYWORDS,
+        .ml_doc     = macos_available_doc
+    },
     {
         .ml_name    = "lookUpClass",
         .ml_meth    = (PyCFunction)lookUpClass,
@@ -1932,6 +2107,12 @@ static PyMethodDef mod_methods[] = {
         .ml_flags   = METH_VARARGS|METH_KEYWORDS,
         .ml_doc     = registerMetaData_doc
     },
+    {
+        .ml_name    = "_copyMetadataRegistry",
+        .ml_meth    = (PyCFunction)copyMetadataRegistry,
+        .ml_flags   = METH_VARARGS|METH_KEYWORDS,
+        .ml_doc     = copyMetadataRegistry_doc
+    },
     {
         .ml_name    = "_updatingMetadata",
         .ml_meth    = (PyCFunction)_updatingMetadata,
@@ -1944,18 +2125,12 @@ static PyMethodDef mod_methods[] = {
         .ml_flags   = METH_VARARGS|METH_KEYWORDS,
         .ml_doc     = _makeClosure_doc
     },
-#if 0
     {
-        .ml_name    = "_sockaddrFromPython",
-        .ml_meth    = (PyCFunction)PyObjC_SockAddrFromPython,
-        .ml_flags   = METH_VARARGS,
+        .ml_name    = "_closurePointer",
+        .ml_meth    = (PyCFunction)_closurePointer,
+        .ml_flags   = METH_VARARGS|METH_KEYWORDS,
+        .ml_doc     = _closurePointer_doc
     },
-    {
-        .ml_name    = "_sockaddrToPython",
-        .ml_meth    = (PyCFunction)PyObjC_SockAddrToPython,
-        .ml_flags   = METH_VARARGS,
-    },
-#endif
     {
         .ml_name    = "_ivar_dict",
         .ml_meth    = (PyCFunction)ivar_dict,
@@ -2098,6 +2273,12 @@ PyObjC_MODULE_INIT(_objc)
 {
     PyObject *m, *d, *v;
 
+    if (PyObjC_Initialized) {
+        PyErr_SetString(PyExc_RuntimeError,
+            "Reload of objc._objc detected, this is not supported");
+        PyObjC_INITERROR();
+    }
+
     PyObjC_SetupRuntimeCompat();
     if (PyErr_Occurred()) {
         PyObjC_INITERROR();
@@ -2178,6 +2359,14 @@ PyObjC_MODULE_INIT(_objc)
     if (PyType_Ready(&PyObjCPythonMethod_Type) < 0) {
         PyObjC_INITERROR();
     }
+    if (PyType_Ready(&StructBase_Type) < 0) {
+        PyObjC_INITERROR();
+    }
+#if PY_MAJOR_VERSION == 3
+    if (PyType_Ready(&FILE_Type) < 0) {
+        PyObjC_INITERROR();
+    }
+#endif
 
 #ifndef Py_HAVE_LOCAL_LOOKUP
     PyObjCSuper_Type.tp_doc = PySuper_Type.tp_doc;
@@ -2234,6 +2423,11 @@ PyObjC_MODULE_INIT(_objc)
     if (PyDict_SetItemString(d, "ObjCPointer", (PyObject*)&PyObjCPointer_Type) < 0) {
         PyObjC_INITERROR();
     }
+#if PY_MAJOR_VERSION == 3
+    if (PyDict_SetItemString(d, "FILE", (PyObject*)&FILE_Type) < 0) {
+        PyObjC_INITERROR();
+    }
+#endif
     if (PyDict_SetItemString(d, "objc_meta_class", (PyObject*)&PyObjCMetaClass_Type) < 0) {
         PyObjC_INITERROR();
     }
@@ -2276,6 +2470,9 @@ PyObjC_MODULE_INIT(_objc)
     if (PyDict_SetItemString(d, "python_method", (PyObject*)&PyObjCPythonMethod_Type) < 0) {
         PyObjC_INITERROR();
     }
+    if (PyDict_SetItemString(d, "_structwrapper", (PyObject*)&StructBase_Type) < 0) {
+        PyObjC_INITERROR();
+    }
 
 #ifndef Py_HAVE_LOCAL_LOOKUP
     if (PyDict_SetItemString(d, "super", (PyObject*)&PyObjCSuper_Type) < 0) {
@@ -2295,7 +2492,7 @@ PyObjC_MODULE_INIT(_objc)
 
 
 #if PyObjC_BUILD_RELEASE >= 1007
-    if (objc_loadWeak != NULL) {
+    if (&objc_loadWeak != NULL) {
         if (PyDict_SetItemString(d, "WeakRef", (PyObject*)&PyObjCWeakRef_Type) < 0) {
             PyObjC_INITERROR();
         }
@@ -2335,7 +2532,7 @@ PyObjC_MODULE_INIT(_objc)
     {
         struct objc_typestr_values* cur = objc_typestr_values;
 
-        for (; cur->name != NULL; cur ++)  {
+        for (; cur->name != NULL; cur ++) {
             PyObject* t = PyBytes_FromStringAndSize(&cur->value, 1);
             if (t == NULL) {
                 PyObjC_INITERROR();
@@ -2383,8 +2580,10 @@ PyObjC_MODULE_INIT(_objc)
 
     PyObjCPointerWrapper_Init();
 
-#if    PyObjC_BUILD_RELEASE >= 1006
+#if PyObjC_BUILD_RELEASE >= 1006
+#if MAC_OS_X_VERSION_MIN_REQUIRED < MAC_OS_X_VERSION_10_6
     if (objc_setAssociatedObject != NULL) {
+#endif /* MAC_OS_X_VERSION_MIN_REQUIRED < MAC_OS_X_VERSION_10_6 */
         if (PyModule_AddIntConstant(m, "OBJC_ASSOCIATION_ASSIGN", OBJC_ASSOCIATION_ASSIGN) < 0) {
             PyObjC_INITERROR();
         }
@@ -2400,6 +2599,7 @@ PyObjC_MODULE_INIT(_objc)
         if (PyModule_AddIntConstant(m, "OBJC_ASSOCIATION_COPY", OBJC_ASSOCIATION_COPY) < 0) {
             PyObjC_INITERROR();
         }
+#if MAC_OS_X_VERSION_MIN_REQUIRED < MAC_OS_X_VERSION_10_6
     } else {
         /* Build on a system where object associations are available, running on a platform where they aren't.
          * Disable the wrappers.
@@ -2415,9 +2615,13 @@ PyObjC_MODULE_INIT(_objc)
         }
 
     }
+#endif /* MAC_OS_X_VERSION_MIN_REQUIRED < MAC_OS_X_VERSION_10_6 */
 #endif /* PyObjC_BUILD_RELEASE >= 1006 */
 
 
+    if (PyModule_AddIntConstant(m, "MAC_OS_X_VERSION_CURRENT", calc_current_version()) < 0) {
+        PyObjC_INITERROR();
+    }
 
 #ifdef MAC_OS_X_VERSION_MAX_ALLOWED
     /* An easy way to check for the MacOS X version we did build for */
@@ -2432,6 +2636,12 @@ PyObjC_MODULE_INIT(_objc)
     }
 #endif /* MAC_OS_X_VERSION_MIN_REQUIRED */
 
+#ifdef MAC_OS_X_VERSION_10_0
+    if (PyModule_AddIntConstant(m, "MAC_OS_X_VERSION_10_0", MAC_OS_X_VERSION_10_0) < 0) {
+        PyObjC_INITERROR();
+    }
+#endif /* MAC_OS_X_VERSION_10_1 */
+
 #ifdef MAC_OS_X_VERSION_10_1
     if (PyModule_AddIntConstant(m, "MAC_OS_X_VERSION_10_1", MAC_OS_X_VERSION_10_1) < 0) {
         PyObjC_INITERROR();
@@ -2480,10 +2690,155 @@ PyObjC_MODULE_INIT(_objc)
     }
 #endif /* MAC_OS_X_VERSION_10_8 */
 
+#ifdef MAC_OS_X_VERSION_10_9
+    if (PyModule_AddIntConstant(m, "MAC_OS_X_VERSION_10_9", MAC_OS_X_VERSION_10_9) < 0) {
+        PyObjC_INITERROR();
+    }
+#endif /* MAC_OS_X_VERSION_10_9 */
+
+#ifdef MAC_OS_X_VERSION_10_10
+    if (PyModule_AddIntConstant(m, "MAC_OS_X_VERSION_10_10", MAC_OS_X_VERSION_10_10) < 0) {
+        PyObjC_INITERROR();
+    }
+#endif /* MAC_OS_X_VERSION_10_10 */
+
+#ifdef MAC_OS_X_VERSION_10_10_2
+    if (PyModule_AddIntConstant(m, "MAC_OS_X_VERSION_10_10_2", MAC_OS_X_VERSION_10_10_2) < 0) {
+        PyObjC_INITERROR();
+    }
+#endif /* MAC_OS_X_VERSION_10_10_2 */
+
+#ifdef MAC_OS_X_VERSION_10_10_3
+    if (PyModule_AddIntConstant(m, "MAC_OS_X_VERSION_10_10_3", MAC_OS_X_VERSION_10_10_3) < 0) {
+        PyObjC_INITERROR();
+    }
+#endif /* MAC_OS_X_VERSION_10_10_3 */
+
+#ifdef MAC_OS_X_VERSION_10_11
+    if (PyModule_AddIntConstant(m, "MAC_OS_X_VERSION_10_11", MAC_OS_X_VERSION_10_11) < 0) {
+        PyObjC_INITERROR();
+    }
+#endif /* MAC_OS_X_VERSION_10_11 */
+
+#ifdef MAC_OS_X_VERSION_10_11_2
+    if (PyModule_AddIntConstant(m, "MAC_OS_X_VERSION_10_11_2", MAC_OS_X_VERSION_10_11_2) < 0) {
+        PyObjC_INITERROR();
+    }
+#endif /* MAC_OS_X_VERSION_10_11_2 */
+
+#ifdef MAC_OS_X_VERSION_10_11_3
+    if (PyModule_AddIntConstant(m, "MAC_OS_X_VERSION_10_11_3", MAC_OS_X_VERSION_10_11_3) < 0) {
+        PyObjC_INITERROR();
+    }
+#endif /* MAC_OS_X_VERSION_10_11_3 */
+
+#ifdef MAC_OS_X_VERSION_10_11_4
+    if (PyModule_AddIntConstant(m, "MAC_OS_X_VERSION_10_11_4", MAC_OS_X_VERSION_10_11_4) < 0) {
+        PyObjC_INITERROR();
+    }
+#endif /* MAC_OS_X_VERSION_10_11_4 */
+
+#ifdef MAC_OS_X_VERSION_10_12
+    if (PyModule_AddIntConstant(m, "MAC_OS_X_VERSION_10_12", MAC_OS_X_VERSION_10_12) < 0) {
+        PyObjC_INITERROR();
+    }
+#endif /* MAC_OS_X_VERSION_10_12 */
+
+#ifdef MAC_OS_X_VERSION_10_12_1
+    if (PyModule_AddIntConstant(m, "MAC_OS_X_VERSION_10_12_1", MAC_OS_X_VERSION_10_12_1) < 0) {
+        PyObjC_INITERROR();
+    }
+#endif /* MAC_OS_X_VERSION_10_12_1 */
+
+#ifdef MAC_OS_X_VERSION_10_12_2
+    if (PyModule_AddIntConstant(m, "MAC_OS_X_VERSION_10_12_2", MAC_OS_X_VERSION_10_12_2) < 0) {
+        PyObjC_INITERROR();
+    }
+#endif /* MAC_OS_X_VERSION_10_12_2 */
+
+#ifdef MAC_OS_X_VERSION_10_12_4
+    if (PyModule_AddIntConstant(m, "MAC_OS_X_VERSION_10_12_4", MAC_OS_X_VERSION_10_12_4) < 0) {
+        PyObjC_INITERROR();
+    }
+#endif /* MAC_OS_X_VERSION_10_12_4 */
+
+#ifdef MAC_OS_X_VERSION_10_13
+    if (PyModule_AddIntConstant(m, "MAC_OS_X_VERSION_10_13", MAC_OS_X_VERSION_10_13) < 0) {
+        PyObjC_INITERROR();
+    }
+#endif /* MAC_OS_X_VERSION_10_13 */
+
+#ifdef MAC_OS_X_VERSION_10_13_1
+    if (PyModule_AddIntConstant(m, "MAC_OS_X_VERSION_10_13_1", MAC_OS_X_VERSION_10_13_1) < 0) {
+        PyObjC_INITERROR();
+    }
+#endif /* MAC_OS_X_VERSION_10_13_1 */
+
+#ifdef MAC_OS_X_VERSION_10_13_2
+    if (PyModule_AddIntConstant(m, "MAC_OS_X_VERSION_10_13_2", MAC_OS_X_VERSION_10_13_2) < 0) {
+        PyObjC_INITERROR();
+    }
+#endif /* MAC_OS_X_VERSION_10_13_2 */
+
+#ifdef MAC_OS_X_VERSION_10_13_3
+    if (PyModule_AddIntConstant(m, "MAC_OS_X_VERSION_10_13_3", MAC_OS_X_VERSION_10_13_3) < 0) {
+        PyObjC_INITERROR();
+    }
+#endif /* MAC_OS_X_VERSION_10_13_3 */
+
+#ifdef MAC_OS_X_VERSION_10_13_4
+    if (PyModule_AddIntConstant(m, "MAC_OS_X_VERSION_10_13_4", MAC_OS_X_VERSION_10_13_4) < 0) {
+        PyObjC_INITERROR();
+    }
+#endif /* MAC_OS_X_VERSION_10_13_4 */
+
+#ifdef MAC_OS_X_VERSION_10_13_5
+    if (PyModule_AddIntConstant(m, "MAC_OS_X_VERSION_10_13_5", MAC_OS_X_VERSION_10_13_5) < 0) {
+        PyObjC_INITERROR();
+    }
+#endif /* MAC_OS_X_VERSION_10_13_5 */
+
+#ifdef MAC_OS_X_VERSION_10_13_6
+    if (PyModule_AddIntConstant(m, "MAC_OS_X_VERSION_10_13_6", MAC_OS_X_VERSION_10_13_6) < 0) {
+        PyObjC_INITERROR();
+    }
+#endif /* MAC_OS_X_VERSION_10_13_6 */
+
+#ifdef MAC_OS_X_VERSION_10_14
+    if (PyModule_AddIntConstant(m, "MAC_OS_X_VERSION_10_14", MAC_OS_X_VERSION_10_14) < 0) {
+        PyObjC_INITERROR();
+    }
+#endif /* MAC_OS_X_VERSION_10_14 */
+
+#ifdef MAC_OS_X_VERSION_10_14_1
+    if (PyModule_AddIntConstant(m, "MAC_OS_X_VERSION_10_14_1", MAC_OS_X_VERSION_10_14_1) < 0) {
+        PyObjC_INITERROR();
+    }
+#endif /* MAC_OS_X_VERSION_10_14 */
+
+    if (PyModule_AddIntConstant(m, "PyObjC_BUILD_RELEASE", PyObjC_BUILD_RELEASE) < 0) {
+        PyObjC_INITERROR();
+    }
+
     if (PyModule_AddIntConstant(m, "_NSNotFound", NSNotFound) < 0) {
         PyObjC_INITERROR();
     }
 
+
+    if ((v = PyFloat_FromDouble(FLT_MIN)) == NULL) {
+        PyObjC_INITERROR();
+    }
+    if (PyModule_AddObject(m, "_FLT_MIN", v) < 0) {
+        PyObjC_INITERROR();
+    }
+
+    if ((v = PyFloat_FromDouble(FLT_MAX)) == NULL) {
+        PyObjC_INITERROR();
+    }
+    if (PyModule_AddObject(m, "_FLT_MAX", v) < 0) {
+        PyObjC_INITERROR();
+    }
+
     if (PyModule_AddStringConstant(m, "platform", "MACOSX") < 0) {
         PyObjC_INITERROR();
     }
@@ -2520,5 +2875,6 @@ PyObjC_MODULE_INIT(_objc)
     [NSUnarchiver decodeClassName:@"OC_PythonString" asClassName:@"OC_PythonUnicode"];
 #endif /* PY_MAJOR_VERSION == 3 */
 
+    PyObjC_Initialized = 1;
     PyObjC_INITDONE();
 }
