@@ -9,6 +9,8 @@ extern PyObject* PyObjCExc_ObjCRevivalWarning;
 extern PyObject* PyObjCExc_LockError;
 extern PyObject* PyObjCExc_BadPrototypeError;
 extern PyObject* PyObjCExc_UnknownPointerError;
+extern PyObject* PyObjCExc_DeprecationWarning;
+extern PyObject* PyObjCExc_ObjCPointerWarning;
 
 extern int PyObjCUtil_Init(PyObject* module);
 
@@ -82,8 +84,9 @@ static inline Py_ssize_t align(Py_ssize_t offset, Py_ssize_t alignment)
 #define SET_FIELD_INCREF(op, value)             \
     do {                                        \
         PyObject* _py_tmp = (PyObject*)(op);    \
-        Py_XINCREF(value);                      \
-        (op) = value;                           \
+        PyObject* _py_val = (PyObject*)(value); \
+        Py_XINCREF(_py_val);                    \
+        (op) = _py_val;                         \
         Py_XDECREF(_py_tmp);                    \
     } while(0)
 
