@@ -5,7 +5,7 @@
  * Central include file for PyObjC.
  */
 
-#define OBJC_VERSION "3.0.4"
+#define OBJC_VERSION "5.1.1"
 
 #define PY_SSIZE_T_CLEAN
 #include <Python.h>
@@ -70,18 +70,25 @@
 #include "pointer-support.h"
 #include "OC_PythonObject.h"
 #include "OC_PythonArray.h"
+#include "OC_BuiltinPythonArray.h"
 #include "OC_PythonData.h"
+#include "OC_BuiltinPythonData.h"
 #include "OC_PythonDictionary.h"
+#include "OC_BuiltinPythonDictionary.h"
 #include "OC_PythonUnicode.h"
+#include "OC_BuiltinPythonUnicode.h"
 
 #if PY_MAJOR_VERSION == 2
 #include "OC_PythonString.h"
+#include "OC_BuiltinPythonString.h"
 #endif
 
 #include "OC_PythonEnumerator.h"
 #include "OC_PythonDate.h"
 #include "OC_PythonNumber.h"
+#include "OC_BuiltinPythonNumber.h"
 #include "OC_PythonSet.h"
+#include "OC_BuiltinPythonSet.h"
 #include "method-signature.h"
 #include "objc-class.h"
 #include "objc-object.h"
@@ -119,12 +126,16 @@
 #include "ctests.h"
 #include "weakref.h"
 
+#if PY_MAJOR_VERSION == 3
+#include "file_wrapper.h"
+#endif
+
 #define PYOBJC_BUILD
 #include "pyobjc-api.h"
 #undef PyObjC_BUILD
 
 #if __has_feature(objc_arc_weak)
-#  error "It is not possible to compile PyObjC with ARC enabled"
+# error "It is not possible to compile PyObjC with ARC enabled"
 #endif
 
 /*
