@@ -9,7 +9,7 @@
 #include <objc/Object.h>
 
 
-static char* pysel_default_signature(PyObject* callable);
+static char* pysel_default_signature(SEL selector, PyObject* callable);
 static PyObject* pysel_new(PyTypeObject* type, PyObject* args, PyObject* kwds);
 
 /*
@@ -17,7 +17,7 @@ static PyObject* pysel_new(PyTypeObject* type, PyObject* args, PyObject* kwds);
  *
  * selectors are callable objects with the following attributes:
  * - 'signature': The objective-C signature of the method
- * - 'selector':  The name in the objective-C runtime
+ * - 'selector': The name in the objective-C runtime
  */
 
 PyObjCMethodSignature*
@@ -25,6 +25,13 @@ PyObjCSelector_GetMetadata(PyObject* _self)
 {
     PyObjCSelector* self = (PyObjCSelector*)_self;
 
+    if (self->sel_methinfo != NULL
+            /* XXX: && PyObjCNativeSelector_Check(self) */
+            && self->sel_mappingcount != PyObjC_MappingCount) {
+        Py_DECREF(self->sel_methinfo);
+        self->sel_methinfo = NULL;
+    }
+
     if (self->sel_methinfo == NULL) {
         self->sel_methinfo = PyObjCMethodSignature_ForSelector(
             self->sel_class,
@@ -55,8 +62,16 @@ PyObjCSelector_GetMetadata(PyObject* _self)
 PyDoc_STRVAR(sel_metadata_doc, "__metadata__()\n" CLINIC_SEP "\nReturn a dict that describes the metadata for this method, including metadata for the 2 hidden ObjC parameters (self and _sel) ");
 static PyObject* sel_metadata(PyObject* self)
 {
-    PyObject* result = PyObjCMethodSignature_AsDict(PyObjCSelector_GetMetadata(self));
     int r;
+    PyObjCMethodSignature* mi = PyObjCSelector_GetMetadata(self);
+    if (mi == NULL) {
+        return NULL;
+    }
+
+    PyObject* result = PyObjCMethodSignature_AsDict(mi);
+    if (result == NULL) {
+        return NULL;
+    }
 
     r = PyDict_SetItemString(result, "classmethod",
         (((PyObjCSelector*)self)->sel_flags & PyObjCSelector_kCLASS_METHOD) ? Py_True : Py_False);
@@ -466,6 +481,8 @@ objcsel_call(PyObject* _self, PyObject* args, PyObject* kwds)
     PyObjC_CallFunc execute = NULL;
     PyObject* res;
     PyObject* pyres;
+    PyObjCMethodSignature* methinfo;
+
 
     if (kwds != NULL && PyObject_Size(kwds) != 0) {
         PyErr_SetString(PyExc_TypeError,
@@ -488,6 +505,28 @@ objcsel_call(PyObject* _self, PyObject* args, PyObject* kwds)
         }
     }
 
+    methinfo = PyObjCSelector_GetMetadata(_self);
+    if (methinfo == NULL) {
+        return NULL;
+    }
+
+    if (PyObjC_DeprecationVersion && methinfo->deprecated && methinfo->deprecated <= PyObjC_DeprecationVersion) {
+        char buf[128];
+        Class cls = PyObjCSelector_GetClass(_self);
+        SEL sel = PyObjCSelector_GetSelector(_self);
+
+        assert(cls);
+        assert(sel);
+
+        snprintf(buf, 128, "%c[%s %s] is a deprecated API (macOS %d.%d)",
+                PyObjCSelector_IsClassMethod(_self) ? '+' : '-',
+                class_getName(cls),
+                sel_getName(sel), methinfo->deprecated / 100, methinfo->deprecated % 100);
+        if (PyErr_Warn(PyObjCExc_DeprecationWarning, buf) < 0) {
+            return NULL;
+        }
+    }
+
     if (self->sel_call_func) {
         execute = self->sel_call_func;
     } else {
@@ -608,8 +647,8 @@ objcsel_descr_get(PyObject* _self, PyObject* obj, PyObject* class)
         }
     }
     result = PyObject_New(PyObjCNativeSelector, &PyObjCNativeSelector_Type);
-    result->base.sel_selector   = meth->base.sel_selector;
-    result->base.sel_python_signature  = PyObjCUtil_Strdup(meth->base.sel_python_signature);
+    result->base.sel_selector  = meth->base.sel_selector;
+    result->base.sel_python_signature = PyObjCUtil_Strdup(meth->base.sel_python_signature);
     if (result->base.sel_python_signature == NULL) {
         Py_DECREF(result);
         return NULL;
@@ -642,6 +681,10 @@ objcsel_descr_get(PyObject* _self, PyObject* obj, PyObject* class)
             meth->sel_call_func = PyObjC_FindCallFunc(meth->base.sel_class,
                 meth->base.sel_selector);
         }
+        if (meth->sel_call_func == NULL) {
+            Py_DECREF(result);
+            return NULL;
+        }
     }
     result->sel_call_func = meth->sel_call_func;
 
@@ -799,7 +842,7 @@ PyObjCSelector_FindNative(PyObject* self, const char* name)
             if (nil != (methsig = [object methodSignatureForSelector:sel])){
                 PyObjCNativeSelector* res;
 
-                res =  (PyObjCNativeSelector*)PyObjCSelector_NewNative(
+                res = (PyObjCNativeSelector*)PyObjCSelector_NewNative(
                     object_getClass(object), sel,
                     PyObjC_NSMethodSignatureToTypeString(methsig,
                         buf, sizeof(buf)), 0);
@@ -889,7 +932,7 @@ PyObjCSelector_New(PyObject* callable,
         if (len > 30 && strcmp(selname+len-30, "DidEnd:returnCode:contextInfo:") == 0) {
             signature = PyObjCUtil_Strdup(gSheetMethodSignature);
         } else {
-            signature = pysel_default_signature(callable);
+            signature = pysel_default_signature(selector, callable);
         }
     } else {
         signature = PyObjCUtil_Strdup(signature);
@@ -907,7 +950,10 @@ PyObjCSelector_New(PyObject* callable,
         return NULL;
     }
 
-    PyObjC_RemoveInternalTypeCodes((char*)result->base.sel_native_signature);
+    if (PyObjC_RemoveInternalTypeCodes((char*)result->base.sel_native_signature) == -1) {
+        Py_DECREF(result);
+        return NULL;
+    }
 
     result->base.sel_self = NULL;
     result->base.sel_class = cls;
@@ -1129,8 +1175,8 @@ compensate_arglist(PyObject* _self, PyObject* args, PyObject* kwds)
                     PyObject* a = PyTuple_GET_ITEM(args, first_arg + i - 2);
                     if (a != Py_None && a != PyObjC_NULL) {
                         PyErr_Format(PyExc_TypeError,
-                            "argument %" PY_FORMAT_SIZE_T "d is an output argument but is passed a value other than None or objc.NULL (%s)",
-                            i-1-first_arg, PyObject_REPR(args));
+                            "argument %" PY_FORMAT_SIZE_T "d is an output argument but is passed a value other than None or objc.NULL",
+                            i-1-first_arg);
                         return NULL;
                     }
                 }
@@ -1267,7 +1313,7 @@ pysel_call(PyObject* _self, PyObject* args, PyObject* kwargs)
      * Assume callable will check arguments
      */
     if (self->base.sel_self == NULL) {
-        result  = PyObject_Call(self->callable, args, kwargs);
+        result = PyObject_Call(self->callable, args, kwargs);
         Py_DECREF(args);
 
     } else {
@@ -1304,15 +1350,16 @@ pysel_call(PyObject* _self, PyObject* args, PyObject* kwargs)
 }
 
 static char*
-pysel_default_signature(PyObject* callable)
+pysel_default_signature(SEL selector, PyObject* callable)
 {
     PyCodeObject* func_code;
-    Py_ssize_t    arg_count;
-    char*         result;
+    Py_ssize_t arg_count;
+    char* result;
     const unsigned char *buffer;
-    Py_ssize_t    buffer_len;
-    Py_ssize_t    i;
-    int           was_none;
+    Py_ssize_t buffer_len;
+    Py_ssize_t i;
+    int was_none;
+    const char* selname = sel_getName(selector);
 
     if (PyFunction_Check(callable)) {
         func_code = (PyCodeObject*)PyFunction_GetCode(callable);
@@ -1326,36 +1373,63 @@ pysel_default_signature(PyObject* callable)
         return NULL;
     }
 
-    arg_count = func_code->co_argcount;
-    if (arg_count < 1) {
-        PyErr_SetString(PyExc_TypeError,
-            "Objective-C callable methods must take at least one argument");
+    if (selname == NULL) {
+        PyErr_SetString(PyExc_ValueError,
+                "Cannot extract string from selector");
         return NULL;
     }
 
+    arg_count = 0;
+
+    selname = strchr(selname, ':');
+    while (selname) {
+        arg_count ++;
+        selname = strchr(selname+1, ':');
+    }
 
     /* arguments + return-type + selector */
-    result = PyMem_Malloc(arg_count+3);
+    result = PyMem_Malloc(arg_count+4);
     if (result == 0) {
         PyErr_NoMemory();
         return NULL;
     }
 
     /* We want: v@:@... (final sequence of arg_count-1 @-chars) */
-    memset(result, _C_ID, arg_count+2);
+    memset(result, _C_ID, arg_count+3);
     result[0] = _C_VOID;
     result[2] = _C_SEL;
-    result[arg_count+2] = '\0';
+    result[arg_count+3] = '\0';
 
     if (PyObject_AsReadBuffer(func_code->co_code, (const void **)&buffer, &buffer_len)) {
         return NULL;
     }
 
     /*
-       Scan bytecode to find return statements.  If any non-bare return
+       Scan bytecode to find return statements. If any non-bare return
        statement exists, then set the return type to @ (id).
+
+       In Python 3.6 the interpreter switched to a 16-bit word-code instead
+       of 8-bit bytecode, hence the two code paths
     */
     was_none = 0;
+
+#if PY_VERSION_HEX >= 0x03060000
+    PyObjC_Assert (buffer_len % 2 == 0, NULL);
+
+    for (i=0; i<buffer_len; i+=2) {
+        int op = buffer[i];
+        if (op == LOAD_CONST && buffer[i+1] == 0) {
+            was_none = 1;
+        } else {
+            if (op == RETURN_VALUE && !was_none) {
+                result[0] = _C_ID;
+                break;
+            }
+            was_none = 0;
+        }
+    }
+
+#else /* PY_VERSION_HEX < 0x03060000 */
     for (i=0; i<buffer_len; ++i) {
         int op = buffer[i];
         if (op == LOAD_CONST && buffer[i+1] == 0 && buffer[i+2] == 0) {
@@ -1373,6 +1447,8 @@ pysel_default_signature(PyObject* callable)
             i += 2;
         }
     }
+#endif /* PY_VERSION_HEX < 0x03060000 */
+
     return result;
 }
 
@@ -1399,10 +1475,17 @@ pysel_default_selector(PyObject* callable)
         strncpy(buf, PyString_AS_STRING(name), sizeof(buf)-1);
 #endif
 
-    } else  {
+    } else {
         return NULL;
     }
 
+    if (buf[strlen(buf)-1] != '_') {
+        /* Method name doesn't end with an underscore,
+         * the default selector cannot be a multi-segment
+         * one. Hence don't try to translate.
+         */
+        return sel_registerName(buf);
+    }
 
     cur = strchr(buf, '_');
     while (cur != NULL) {
@@ -1606,9 +1689,9 @@ pysel_descr_get(PyObject* _meth, PyObject* obj, PyObject* class)
     }
 
     result = PyObject_New(PyObjCPythonSelector, &PyObjCPythonSelector_Type);
-    result->base.sel_selector   = meth->base.sel_selector;
-    result->base.sel_class   = meth->base.sel_class;
-    result->base.sel_python_signature  = PyObjCUtil_Strdup(meth->base.sel_python_signature);
+    result->base.sel_selector = meth->base.sel_selector;
+    result->base.sel_class = meth->base.sel_class;
+    result->base.sel_python_signature = PyObjCUtil_Strdup(meth->base.sel_python_signature);
 
     if (result->base.sel_python_signature == NULL) {
         Py_DECREF(result);
@@ -1616,7 +1699,7 @@ pysel_descr_get(PyObject* _meth, PyObject* obj, PyObject* class)
     }
 
     if (meth->base.sel_native_signature) {
-        result->base.sel_native_signature  = PyObjCUtil_Strdup(meth->base.sel_native_signature);
+        result->base.sel_native_signature = PyObjCUtil_Strdup(meth->base.sel_native_signature);
         if (result->base.sel_native_signature == NULL) {
             Py_DECREF(result);
             return NULL;
@@ -1627,11 +1710,15 @@ pysel_descr_get(PyObject* _meth, PyObject* obj, PyObject* class)
     }
 
     result->base.sel_methinfo = PyObjCSelector_GetMetadata((PyObject*)meth);
-    Py_XINCREF(result->base.sel_methinfo);
+    if (result->base.sel_methinfo == NULL) {
+        PyErr_Clear();
+    } else {
+        Py_INCREF(result->base.sel_methinfo);
+    }
     result->argcount = meth->argcount;
     result->numoutput = meth->numoutput;
 
-    result->base.sel_self       = obj;
+    result->base.sel_self = obj;
     result->base.sel_flags = meth->base.sel_flags;
     result->callable = meth->callable;
 
@@ -1746,22 +1833,22 @@ PyObjCSelector_GetSelector(PyObject* sel)
 }
 
 
-int   PyObjCSelector_Required(PyObject* obj)
+int PyObjCSelector_Required(PyObject* obj)
 {
     return (((PyObjCSelector*)obj)->sel_flags & PyObjCSelector_kREQUIRED) != 0;
 }
 
-int   PyObjCSelector_IsClassMethod(PyObject* obj)
+int PyObjCSelector_IsClassMethod(PyObject* obj)
 {
     return (PyObjCSelector_GetFlags(obj) & PyObjCSelector_kCLASS_METHOD) != 0;
 }
 
-int   PyObjCSelector_IsHidden(PyObject* obj)
+int PyObjCSelector_IsHidden(PyObject* obj)
 {
     return (PyObjCSelector_GetFlags(obj) & PyObjCSelector_kHIDDEN) != 0;
 }
 
-int   PyObjCSelector_GetFlags(PyObject* obj)
+int PyObjCSelector_GetFlags(PyObject* obj)
 {
     return ((PyObjCSelector*)obj)->sel_flags;
 }
@@ -1861,8 +1948,8 @@ PyObjCSelector_FromFunction(
         result->argcount = ((PyObjCPythonSelector*)callable)->argcount;
         result->base.sel_methinfo = PyObjCSelector_GetMetadata(callable);
         Py_XINCREF(result->base.sel_methinfo);
-        result->base.sel_class   = oc_class;
-        result->base.sel_python_signature  = PyObjCUtil_Strdup(
+        result->base.sel_class = oc_class;
+        result->base.sel_python_signature = PyObjCUtil_Strdup(
                 ((PyObjCPythonSelector*)callable)->base.sel_python_signature);
         if (result->base.sel_python_signature == NULL) {
             Py_DECREF(result);
