@@ -21,7 +21,7 @@ GET_STRUCT_FIELD(PyObject* self, PyMemberDef* member)
 {
     PyObject* v;
 
-    v =  *(PyObject**)(((char*)self) + member->offset);
+    v = *(PyObject**)(((char*)self) + member->offset);
     if (v == NULL) {
         return Py_None;
     } else {
@@ -40,6 +40,13 @@ SET_STRUCT_FIELD(PyObject* self, PyMemberDef* member, PyObject* val)
     Py_XDECREF(tmp);
 }
 
+
+static inline Py_ssize_t
+STRUCT_LENGTH(PyObject* self)
+{
+    return (Py_TYPE(self)->tp_basicsize - sizeof(PyObject)) / sizeof(PyObject*);
+}
+
 /*
  * Implementation of the sequence interface.
  */
@@ -50,19 +57,32 @@ struct_sq_length(PyObject* self)
     /* The object contains the generic PyObject header followed by an
      * array of PyObject*-s.
      */
-    return (Py_TYPE(self)->tp_basicsize - sizeof(PyObject)) / sizeof(PyObject*);
+    if (!PyObjC_StructsIndexable) {
+        PyErr_Format(PyExc_TypeError,
+            "Instances of '%.100s' are not sequences 1", Py_TYPE(self)->tp_name);
+        return -1;
+    }
+    return STRUCT_LENGTH(self);
 }
 
 static PyObject*
 struct_sq_item(PyObject* self, Py_ssize_t offset)
 {
-    Py_ssize_t len = struct_sq_length(self);
+    Py_ssize_t len;
     PyMemberDef* member;
     PyObject* res;
 
+    if (!PyObjC_StructsIndexable) {
+        PyErr_Format(PyExc_TypeError,
+            "Instances of '%.100s' are not sequences 2", Py_TYPE(self)->tp_name);
+        return NULL;
+    }
+
+    len = STRUCT_LENGTH(self);
+
     if (offset < 0 || offset >= len) {
         PyErr_Format(PyExc_IndexError,
-                "%s index out of range",
+                "%.100s index out of range",
                 Py_TYPE(self)->tp_name);
         return NULL;
     }
@@ -80,8 +100,14 @@ struct_sq_slice(PyObject* self, Py_ssize_t ilow, Py_ssize_t ihigh)
     PyObject* result;
     Py_ssize_t i, len;
 
-    len = struct_sq_length(self);
-    if (ilow < 0)  ilow = 0;
+    if (!PyObjC_StructsIndexable) {
+        PyErr_Format(PyExc_TypeError,
+            "Instances of '%.100s' are not sequences 3", Py_TYPE(self)->tp_name);
+        return NULL;
+    }
+
+    len = STRUCT_LENGTH(self);
+    if (ilow < 0) ilow = 0;
     if (ihigh > len) ihigh = len;
 
     result = PyTuple_New(ihigh - ilow);
@@ -104,18 +130,29 @@ struct_sq_ass_item(PyObject* self, Py_ssize_t offset, PyObject* newVal)
     Py_ssize_t len;
     PyMemberDef* member;
 
+    if (!PyObjC_StructsIndexable) {
+        PyErr_Format(PyExc_TypeError,
+            "Instances of '%.100s' are not sequences 4", Py_TYPE(self)->tp_name);
+        return -1;
+    }
+    if (!PyObjC_StructsWritable) {
+        PyErr_Format(PyExc_TypeError,
+            "Instances of '%.100s' are read-only", Py_TYPE(self)->tp_name);
+        return -1;
+    }
+
     if (newVal == NULL) {
         PyErr_Format(PyExc_TypeError,
-            "Cannot delete item '%"PY_FORMAT_SIZE_T"d' in a %s instance",
+            "Cannot delete item '%"PY_FORMAT_SIZE_T"d' in a %.100s instance",
             offset, Py_TYPE(self)->tp_name);
         return -1;
     }
 
-    len = struct_sq_length(self);
+    len = STRUCT_LENGTH(self);
 
     if ((offset < 0) || (offset >= len)) {
         PyErr_Format(PyExc_IndexError,
-                "%s index out of range",
+                "%.100s index out of range",
                 Py_TYPE(self)->tp_name);
         return -1;
     }
@@ -130,16 +167,27 @@ struct_sq_ass_slice(PyObject* self, Py_ssize_t ilow, Py_ssize_t ihigh, PyObject*
     PyObject* seq;
     Py_ssize_t i, len;
 
+    if (!PyObjC_StructsIndexable) {
+        PyErr_Format(PyExc_TypeError,
+            "Instances of '%.100s' are not sequences 5", Py_TYPE(self)->tp_name);
+        return -1;
+    }
+    if (!PyObjC_StructsWritable) {
+        PyErr_Format(PyExc_TypeError,
+            "Instances of '%.100s' are read-only", Py_TYPE(self)->tp_name);
+        return -1;
+    }
+
     if (v == NULL) {
         PyErr_Format(PyExc_TypeError,
-            "Cannot delete items in an %s instance",
+            "Cannot delete items in instances of %.100s",
             Py_TYPE(self)->tp_name);
         return -1;
     }
 
 
-    len = struct_sq_length(self);
-    if (ilow < 0)  {
+    len = STRUCT_LENGTH(self);
+    if (ilow < 0) {
         ilow = 0;
     } else if (ilow > len) {
         ilow = len;
@@ -157,7 +205,7 @@ struct_sq_ass_slice(PyObject* self, Py_ssize_t ilow, Py_ssize_t ihigh, PyObject*
     if (PySequence_Fast_GET_SIZE(seq) != ihigh - ilow) {
         Py_DECREF(seq);
         PyErr_Format(PyExc_TypeError,
-            "slice assignment would change size of %s "
+            "slice assignment would change size of %.100s "
             "instance", Py_TYPE(self)->tp_name);
         return -1;
     }
@@ -182,6 +230,12 @@ struct_sq_contains(PyObject* self, PyObject* value)
 {
     PyMemberDef* member = Py_TYPE(self)->tp_members;
 
+    if (!PyObjC_StructsIndexable) {
+        PyErr_Format(PyExc_TypeError,
+            "Instances of '%.100s' are not sequences", Py_TYPE(self)->tp_name);
+        return -1;
+    }
+
     while (member && member->name) {
         int r;
         PyObject* cur = GET_STRUCT_FIELD(self, member);
@@ -203,7 +257,7 @@ struct_reduce(PyObject* self)
     PyObject* values;
     Py_ssize_t i, len;
 
-    len = struct_sq_length(self);
+    len = STRUCT_LENGTH(self);
     values = PyTuple_New(len);
     if (values == NULL) return NULL;
 
@@ -341,6 +395,12 @@ struct_asdict(PyObject* self)
 static PyObject*
 struct_mp_subscript(PyObject* self, PyObject* item)
 {
+    if (!PyObjC_StructsIndexable) {
+        PyErr_Format(PyExc_TypeError,
+            "Instances of '%.100s' are not sequences 7", Py_TYPE(self)->tp_name);
+        return NULL;
+    }
+
     if (PyIndex_Check(item)) {
         Py_ssize_t i;
         i = PyNumber_AsSsize_t(item, PyExc_IndexError);
@@ -350,7 +410,7 @@ struct_mp_subscript(PyObject* self, PyObject* item)
         }
 
         if (i < 0) {
-            i += struct_sq_length(self);
+            i += STRUCT_LENGTH(self);
         }
         return struct_sq_item(self, i);
 
@@ -360,7 +420,7 @@ struct_mp_subscript(PyObject* self, PyObject* item)
         PyObject* it;
 
         if (PySlice_GetIndicesEx(SLICE_CAST(item),
-                struct_sq_length(self),
+                STRUCT_LENGTH(self),
                 &start, &stop, &step, &slicelength) < 0) {
             return NULL;
         }
@@ -387,7 +447,7 @@ struct_mp_subscript(PyObject* self, PyObject* item)
 
     } else {
         PyErr_Format(PyExc_TypeError,
-            "struct indices must be integers, not %.200s",
+            "struct indices must be integers, not %.100s",
             Py_TYPE(item)->tp_name);
         return NULL;
     }
@@ -396,6 +456,17 @@ struct_mp_subscript(PyObject* self, PyObject* item)
 static int
 struct_mp_ass_subscript(PyObject* self, PyObject* item, PyObject* value)
 {
+    if (!PyObjC_StructsIndexable) {
+        PyErr_Format(PyExc_TypeError,
+            "Instances of '%.100s' are not sequences 8", Py_TYPE(self)->tp_name);
+        return -1;
+    }
+    if (!PyObjC_StructsWritable) {
+        PyErr_Format(PyExc_TypeError,
+            "Instances of '%.100s' are read-only", Py_TYPE(self)->tp_name);
+        return -1;
+    }
+
     if (PyIndex_Check(item)) {
         Py_ssize_t i = PyNumber_AsSsize_t(item, PyExc_IndexError);
 
@@ -404,14 +475,14 @@ struct_mp_ass_subscript(PyObject* self, PyObject* item, PyObject* value)
         }
 
         if (i < 0) {
-            i += struct_sq_length(self);
+            i += STRUCT_LENGTH(self);
         }
         return struct_sq_ass_item(self, i, value);
     } else if (PySlice_Check(item)) {
         Py_ssize_t start, stop, step, slicelength;
 
         if (PySlice_GetIndicesEx(SLICE_CAST(item),
-                struct_sq_length(self), &start, &stop,
+                STRUCT_LENGTH(self), &start, &stop,
                 &step, &slicelength) < 0) {
             return -1;
         }
@@ -422,7 +493,7 @@ struct_mp_ass_subscript(PyObject* self, PyObject* item, PyObject* value)
 
         if (value == NULL) {
             PyErr_Format(PyExc_TypeError,
-                "Cannot delete items in an %s instance",
+                "Cannot delete items in an %.100s instance",
                 Py_TYPE(self)->tp_name);
             return -1;
         }
@@ -434,7 +505,7 @@ struct_mp_ass_subscript(PyObject* self, PyObject* item, PyObject* value)
         if (PySequence_Fast_GET_SIZE(seq) != slicelength) {
             Py_DECREF(seq);
             PyErr_Format(PyExc_TypeError,
-                "slice assignment would change size of %s "
+                "slice assignment would change size of %.100s "
                 "instance", Py_TYPE(self)->tp_name);
             return -1;
         }
@@ -454,7 +525,7 @@ struct_mp_ass_subscript(PyObject* self, PyObject* item, PyObject* value)
 
     } else {
         PyErr_Format(PyExc_TypeError,
-            "struct indices must be integers, not %.200s",
+            "struct indices must be integers, not %.100s",
             Py_TYPE(item)->tp_name);
         return -1;
     }
@@ -524,8 +595,13 @@ static PyMethodDef struct_methods[] = {
 static int
 struct_setattro(PyObject* self, PyObject* name, PyObject* value)
 {
+    if (!PyObjC_StructsWritable) {
+        PyErr_Format(PyExc_TypeError,
+            "Instances of '%.100s' are read-only", Py_TYPE(self)->tp_name);
+        return -1;
+    }
     if (value == NULL) {
-        PyErr_Format(PyExc_TypeError, "Cannot delete attributes of %s",
+        PyErr_Format(PyExc_TypeError, "Cannot delete attributes of %.100s",
                 Py_TYPE(self)->tp_name);
         return -1;
     }
@@ -580,7 +656,7 @@ static int LOCATE_MEMBER(PyTypeObject* type, const char* name)
     int i = 0;
     PyMemberDef* member;
 
-    for (i = 0,  member = type->tp_members;
+    for (i = 0, member = type->tp_members;
             member->name != NULL; i++, member++) {
         if (strcmp(member->name, name) == 0) {
             return i;
@@ -676,12 +752,11 @@ static int set_defaults(PyObject* self, const char* typestr)
             return -1;
         }
 
-        r = PySequence_SetItem(self, i++, v);
+        r = PyObjC_SetStructField(self, i++, v);
         Py_DECREF(v);
-        if (r != 0) {
+        if (r < 0) {
             return -1;
         }
-
         typestr = next;
     }
 
@@ -711,7 +786,7 @@ struct_init(
 
     if (args != NULL && !PyTuple_Check(args)) {
         PyErr_Format(PyExc_TypeError,
-                "%s() argument tuple is not a tuple",
+                "%.100s() argument tuple is not a tuple",
                 Py_TYPE(self)->tp_name);
         *(int*)retval = -1;
         return;
@@ -719,7 +794,7 @@ struct_init(
 
     if (kwds != NULL && !PyDict_Check(kwds)) {
         PyErr_Format(PyExc_TypeError,
-                "%s() keyword dict is not a dict",
+                "%.100s() keyword dict is not a dict",
                 Py_TYPE(self)->tp_name);
         *(int*)retval = -1;
         return;
@@ -735,11 +810,11 @@ struct_init(
         Py_ssize_t i, len;
 
         len = PyTuple_GET_SIZE(args);
-        if (len > struct_sq_length(self)) {
+        if (len > STRUCT_LENGTH(self)) {
             PyErr_Format(PyExc_TypeError,
-                "%s() takes at most %"PY_FORMAT_SIZE_T"d %sarguments (%"PY_FORMAT_SIZE_T"d given)",
+                "%.100s() takes at most %"PY_FORMAT_SIZE_T"d %sarguments (%"PY_FORMAT_SIZE_T"d given)",
                 Py_TYPE(self)->tp_name,
-                struct_sq_length(self),
+                STRUCT_LENGTH(self),
                 kwds?"non-keyword ":"", len);
             *(int*)retval = -1;
             return;
@@ -794,7 +869,7 @@ struct_init(
             } else {
                 Py_DECREF(keys);
                 PyErr_Format(PyExc_TypeError,
-                    "%s() keywords must be strings",
+                    "%.100s() keywords must be strings",
                     Py_TYPE(self)->tp_name);
                 *(int*)retval = -1;
                 return;
@@ -805,7 +880,7 @@ struct_init(
                     PyBytes_AS_STRING(k_bytes));
             if (off == -1) {
                 PyErr_Format(PyExc_TypeError,
-                    "no keyword argument: %s",
+                    "no keyword argument: %.100s",
                     PyBytes_AS_STRING(k_bytes));
                 Py_DECREF(k_bytes);
                 Py_DECREF(keys);
@@ -815,8 +890,8 @@ struct_init(
 
             if (off <= setUntil) {
                 PyErr_Format(PyExc_TypeError,
-                    "%s() got multiple values for keyword "
-                    "argument '%s'",
+                    "%.100s() got multiple values for keyword "
+                    "argument '%.100s'",
                     Py_TYPE(self)->tp_name,
                     PyBytes_AS_STRING(k_bytes));
                 Py_DECREF(k_bytes);
@@ -844,24 +919,32 @@ static ffi_cif* init_cif = NULL;
     ffi_closure* cl = NULL;
     ffi_status rv;
 
+    typestr = PyObjCUtil_Strdup(typestr);
+    if (typestr == NULL) {
+        return NULL;
+    }
+
     if (init_cif == NULL) {
         PyObjCMethodSignature* signature;
         signature = PyObjCMethodSignature_FromSignature("i^v^v^v", YES);
         init_cif = PyObjCFFI_CIFForSignature(signature);
         Py_DECREF(signature);
         if (init_cif == NULL) {
+            PyMem_Free((void*)typestr);
             return NULL;
         }
     }
 
     cl = PyObjC_malloc_closure();
     if (cl == NULL) {
+        PyMem_Free((void*)typestr);
         return NULL;
     }
 
     rv = ffi_prep_closure(cl, init_cif, struct_init, (char*)typestr);
     if (rv != FFI_OK) {
         PyObjC_free_closure(cl);
+        PyMem_Free((void*)typestr);
         PyErr_Format(PyExc_RuntimeError,
             "Cannot create FFI closure: %d", rv);
         return NULL;
@@ -873,7 +956,7 @@ static ffi_cif* init_cif = NULL;
 static long
 struct_hash(PyObject* self)
 {
-    PyErr_Format(PyExc_TypeError, "%s objects are unhashable",
+    PyErr_Format(PyExc_TypeError, "%.100s objects are unhashable",
             Py_TYPE(self)->tp_name);
     return -1;
 }
@@ -886,6 +969,63 @@ struct_richcompare(PyObject* self, PyObject* other, int op)
     PyObject* self_cur;
     PyObject* other_cur;
 
+    if (Py_TYPE(self) == Py_TYPE(other)) {
+        /* Other has same type, shortcut comparisions to avoid
+         * treating "other" as a generic sequence
+         */
+
+        len = STRUCT_LENGTH(self);
+
+        for (i = 0; i < len; i ++) {
+            int k;
+
+            self_cur = GET_STRUCT_FIELD(self, Py_TYPE(self)->tp_members+i);
+            other_cur = GET_STRUCT_FIELD(other, Py_TYPE(other)->tp_members+i);
+
+            k = PyObject_RichCompareBool(self_cur, other_cur, Py_EQ);
+            if (k < 0) {
+                return NULL;
+            }
+
+            if (!k) {
+                /* Not equal, result is the comparison of the last
+                 * item, we can do better for '==' and '!='.
+                 */
+                PyObject* v;
+
+                if (op == Py_EQ) {
+                    Py_INCREF(Py_False);
+                    return Py_False;
+                } else if (op == Py_NE) {
+                    Py_INCREF(Py_True);
+                    return Py_True;
+                }
+                v = PyObject_RichCompare(self_cur, other_cur, op);
+                return v;
+            }
+        }
+
+        /* All items are equal, compare using sizes */
+        switch (op) {
+        case Py_LT:
+        case Py_NE:
+        case Py_GT:
+            Py_INCREF(Py_False);
+            return Py_False;
+
+        case Py_LE:
+        case Py_EQ:
+        case Py_GE:
+            Py_INCREF(Py_True);
+            return Py_True;
+
+        default:
+            /* Should never happen */
+            PyErr_SetString(PyExc_TypeError, "Invalid comparion");
+            return NULL;
+        }
+    }
+
     if (!PySequence_Check(other)) {
         if (op == Py_EQ) {
             Py_INCREF(Py_False);
@@ -897,14 +1037,31 @@ struct_richcompare(PyObject* self, PyObject* other, int op)
 
         } else {
             PyErr_Format(PyExc_TypeError,
-                "Cannot compare instances of %s and %s",
+                "Cannot compare instances of %.100s and %.100s",
+                Py_TYPE(self)->tp_name,
+                Py_TYPE(other)->tp_name);
+            return NULL;
+        }
+
+    } else if (!PyObjC_StructsIndexable) {
+        if (op == Py_EQ) {
+            Py_INCREF(Py_False);
+            return Py_False;
+
+        } else if (op == Py_NE) {
+            Py_INCREF(Py_True);
+            return Py_True;
+
+        } else {
+            PyErr_Format(PyExc_TypeError,
+                "Cannot compare instances of %.100s and %.100s",
                 Py_TYPE(self)->tp_name,
                 Py_TYPE(other)->tp_name);
             return NULL;
         }
     }
 
-    self_len = struct_sq_length(self);
+    self_len = STRUCT_LENGTH(self);
     other_len = PySequence_Length(other);
     len = self_len;
     if (other_len < len) {
@@ -1018,9 +1175,9 @@ struct_repr(PyObject* self)
     PyObject* cur;
     PyMemberDef* member;
 
-    len = struct_sq_length(self);
+    len = STRUCT_LENGTH(self);
     if (len == 0) {
-        return PyText_FromFormat("<%s>",
+        return PyText_FromFormat("<%.100s>",
                 Py_TYPE(self)->tp_name);
     }
 
@@ -1030,18 +1187,18 @@ struct_repr(PyObject* self)
 
     } else if (i != 0) {
         /* Self-recursive struct */
-        return PyText_FromFormat("<%s ...>",
+        return PyText_FromFormat("<%.100s ...>",
                 Py_TYPE(self)->tp_name);
     }
 
-    cur = PyText_FromFormat("<%s", Py_TYPE(self)->tp_name);
+    cur = PyText_FromFormat("<%.100s", Py_TYPE(self)->tp_name);
 
     member = Py_TYPE(self)->tp_members;
     while (member->name != NULL) {
         PyObject* v;
 
         PyText_Append(&cur,
-            PyText_FromFormat(" %s=", member->name));
+            PyText_FromFormat(" %.100s=", member->name));
         if (cur == NULL) goto done;
 
         v = GET_STRUCT_FIELD(self, member);
@@ -1058,9 +1215,18 @@ done:
     return cur;
 }
 
+
+PyTypeObject StructBase_Type = {
+    PyVarObject_HEAD_INIT(NULL, 0)
+    .tp_name    = "objc._structwrapper",
+    .tp_basicsize   = sizeof(PyObject),
+    .tp_itemsize    = 0,
+};
+
+
 struct StructTypeObject {
-    PyTypeObject     base;
-    Py_ssize_t       pack;         /* struct packing, -1 for default packing */
+    PyTypeObject base;
+    Py_ssize_t pack;         /* struct packing, -1 for default packing */
 };
 
 /*
@@ -1184,12 +1350,16 @@ PyObjC_MakeStructType(
 
     result->pack = pack;
 
+    result->base.tp_base = &StructBase_Type;
+    Py_INCREF(result->base.tp_base);
+
     if (PyType_Ready((PyTypeObject*)result) == -1) {
         PyMem_Free(result);
         PyMem_Free(members);
         return NULL;
     }
 
+
     return (PyObject*)result;
 }
 
@@ -1411,7 +1581,7 @@ PyObjC_RegisterStructType(
 
     if (pack != -1) {
         /* Store custom struct packing as an attribute of the type
-         * object, to be able to  fetch it when depythonifying the object.
+         * object, to be able to fetch it when depythonifying the object.
          */
         v = Py_BuildValue("n", pack);
         if (v == NULL) {
@@ -1445,7 +1615,9 @@ PyObjC_RegisterStructType(
     }
 
     /* Register again using the typecode used in the ObjC runtime */
-    PyObjC_RemoveInternalTypeCodes((char*)signature);
+    if (PyObjC_RemoveInternalTypeCodes((char*)signature) == -1) {
+        return NULL;
+    }
     r = PyDict_SetItemString(structRegistry, signature, structType);
     if (r == -1) {
         return NULL;
@@ -1481,7 +1653,9 @@ PyObjC_RegisterStructAlias(const char* signature, PyObject* structType)
     }
 
     /* Register again using the typecode used in the ObjC runtime */
-    PyObjC_RemoveInternalTypeCodes(buf);
+    if (PyObjC_RemoveInternalTypeCodes(buf) == -1) {
+        return -1;
+    }
     r = PyDict_SetItemString(structRegistry, buf, structType);
     if (r == -1) {
         return -1;
@@ -1489,3 +1663,47 @@ PyObjC_RegisterStructAlias(const char* signature, PyObject* structType)
 
     return 0;
 }
+
+int
+PyObjC_SetStructField(PyObject* self, Py_ssize_t offset, PyObject* newVal)
+{
+    Py_ssize_t len;
+    PyMemberDef* member;
+
+    if (newVal == NULL) {
+        PyErr_Format(PyExc_TypeError,
+            "Cannot delete item '%"PY_FORMAT_SIZE_T"d' in a %.100s instance",
+            offset, Py_TYPE(self)->tp_name);
+        return -1;
+    }
+
+    len = STRUCT_LENGTH(self);
+
+    if ((offset < 0) || (offset >= len)) {
+        PyErr_Format(PyExc_IndexError,
+                "%.100s index out of range",
+                Py_TYPE(self)->tp_name);
+        return -1;
+    }
+    member = Py_TYPE(self)->tp_members + offset;
+    SET_STRUCT_FIELD(self, member, newVal);
+    return 0;
+}
+
+PyObject*
+StructAsTuple(PyObject* strval)
+{
+    Py_ssize_t i, len = STRUCT_LENGTH(strval);
+    PyObject* retval = PyTuple_New(len);
+    if (retval == NULL) {
+        return 0;
+    }
+
+    for (i = 0; i < len; i++) {
+        PyObject* v;
+        v = GET_STRUCT_FIELD(strval, Py_TYPE(strval)->tp_members+i);
+        PyTuple_SET_ITEM(retval, i, v);
+        Py_INCREF(v);
+    }
+    return retval;
+}
