@@ -65,6 +65,33 @@ objc.parseBridgeSupport('''\
               <arg type='d' />
           </arg>
         </method>
+        <method selector='signatureForBlock1:'>
+          <arg index='0' block='true'>
+           <retval type='d' />
+           <arg type='d' />
+           <arg type='d' />
+          </arg>
+        </method>
+        <method selector='signatureForBlock2:'>
+          <arg index='0' block='true'>
+           <retval type='@' />
+           <arg type='@' />
+          </arg>
+        </method>
+        <method selector='signatureForBlock3:'>
+          <arg index='0' block='true'>
+           <retval type='@' />
+           <arg type='s' />
+          </arg>
+        </method>
+        <method selector='signatureForBlock4:'>
+          <arg index='0' block='true'>
+           <retval type='c' />
+           <arg type='i' />
+           <arg type='i' />
+           <arg type='f' />
+          </arg>
+        </method>
       </class>
       <class name='NSObject'>
         <method selector='processBlock:'>
@@ -75,6 +102,12 @@ objc.parseBridgeSupport('''\
             <arg type='d' />
           </arg>
         </method>
+        <method selector='callWithCompletion:'>
+          <arg index='0' block='true' type='@?'>
+            <retval type='v'/>
+            <arg type='@' />
+          </arg>
+        </method>
         <method selector='optionalBlock:'>
           <retval type='@' />
           <arg index='0' block='true' type='@?'>
@@ -108,6 +141,19 @@ class BlocksHelper (objc.lookUpClass('NSObject')):
             return block("x")
 
 
+class BlocksCompletion (objc.lookUpClass('NSObject')):
+    def callWithCompletion_(self, completion):
+        completion('hello')
+        completion('world')
+
+class BlockWithStoredCompletion (objc.lookUpClass('NSObject')):
+    def callWithCompletion_(self, completion):
+        self.completion = completion
+
+    def performCompletions(self):
+        self.completion('hello')
+        self.completion('world')
+
 class TestBlocks (TestCase):
     @min_os_level('10.6')
     @onlyIf(blocksEnabled, "no blocks")
@@ -167,6 +213,30 @@ class TestBlocks (TestCase):
         self.assertEqual(len(helper.values), 2)
         self.assertEqual(helper.values, [42, 43])
 
+    @min_os_level('10.6')
+    @onlyIf(blocksEnabled, "no blocks")
+    def testStackBlocksWithDirectUse(self):
+        obj = OCTestBlock.alloc().init()
+        tester = BlocksCompletion.alloc().init()
+        a = []
+
+        obj.callCompletionOn_andArray_(tester, a)
+        self.assertEqual(a, ['hello', 'world'])
+
+    @min_os_level('10.6')
+    @onlyIf(blocksEnabled, "no blocks")
+    def testStackBlocksWithIndirectUse(self):
+        obj = OCTestBlock.alloc().init()
+        tester = BlockWithStoredCompletion.alloc().init()
+        a = []
+
+        obj.callCompletionOn_andArray_(tester, a)
+        self.assertEqual(a, [])
+
+        tester.performCompletions()
+
+        self.assertEqual(a, ['hello', 'world'])
+
     @min_os_level('10.6')
     @onlyIf(blocksEnabled, "no blocks")
     def testBlockToObjC2(self):
@@ -271,6 +341,33 @@ class TestBlocks (TestCase):
         value = block("hello", "world");
         self.assertEqual(value, 10)
 
+BLOCK_SELF_TYPE=objc._C_PTR + objc._C_VOID
+class TestBlockRuntimeSignature (TestCase):
+    @onlyIf(blocksEnabled, "no blocks")
+    def testBlock1(self):
+        obj = OCTestBlock.alloc().init()
+        signature = obj.signatureForBlock1_(lambda a, b: a*b)
+        self.assertEqual(signature, (objc._C_DBL + BLOCK_SELF_TYPE + objc._C_DBL + objc._C_DBL).decode('utf-8'))
+
+    @onlyIf(blocksEnabled, "no blocks")
+    def testBlock2(self):
+        obj = OCTestBlock.alloc().init()
+        signature = obj.signatureForBlock2_(lambda a: a)
+        self.assertEqual(signature, (objc._C_ID + BLOCK_SELF_TYPE + objc._C_ID).decode('utf-8'))
+
+    @onlyIf(blocksEnabled, "no blocks")
+    def testBlock3(self):
+        obj = OCTestBlock.alloc().init()
+        signature = obj.signatureForBlock3_(lambda a: a)
+        self.assertEqual(signature, (objc._C_ID + BLOCK_SELF_TYPE + objc._C_SHT).decode('utf-8'))
+
+    @onlyIf(blocksEnabled, "no blocks")
+    def testBlock4(self):
+        obj = OCTestBlock.alloc().init()
+        signature = obj.signatureForBlock4_(lambda a, b, c: a)
+        self.assertEqual(signature, (objc._C_CHR + BLOCK_SELF_TYPE + objc._C_INT + objc._C_INT + objc._C_FLT).decode('utf-8'))
+
+
 
 if __name__ == "__main__":
     main()
