@@ -8,20 +8,10 @@ import imp
 import ctypes
 import objc
 import subprocess
+import warnings
 
 import xml.etree.ElementTree as ET
 
-
-try:
-    basestring
-except NameError:
-    basestring = str
-
-try:
-    long
-except NameError:
-    long = int
-
 IDENTIFIER=re.compile(r"^[A-Za-z_][A-Za-z0-9_]*$")
 
 TEST_XML=b"""\
@@ -439,35 +429,63 @@ TEST_XML=b"""\
 </signatures>
 """
 
+def iter_framework_dir(framework_dir):
+    for dn in os.listdir(framework_dir):
+        fn = os.path.join(framework_dir, dn, 'Resources', 'BridgeSupport', dn.replace('.framework', '.bridgesupport'))
+        if os.path.exists(fn):
+            yield fn
+
+        fn = os.path.join(framework_dir, dn, 'Frameworks')
+        if os.path.exists(fn):
+            for item in iter_framework_dir(fn):
+                yield item
+
+def iter_system_bridgesupport_files():
+    for item in iter_framework_dir('/System/Library/Frameworks'):
+        yield item
+
+def contains_any(name, fragments):
+    return any(x in name for x in fragments)
+
 class TestBridgeSupportParser (TestCase):
     def testInvalidToplevel(self):
         self.assertRaises(objc.error, bridgesupport._BridgeSupportParser, b'<signatures2></signatures2>', 'Cocoa')
         self.assertRaises(ET.ParseError, bridgesupport._BridgeSupportParser, b'<signatures2></signatures>', 'Cocoa')
 
-    def iter_framework_dir(self, framework_dir):
-        for dn in os.listdir(framework_dir):
-            fn = os.path.join(framework_dir, dn, 'Resources', 'BridgeSupport', dn.replace('.framework', '.bridgesupport'))
-            if os.path.exists(fn):
-                yield fn
+    # I'd like to use a test method with subTests here, but that doesn't support marking some subtests as expected failures
+    BROKEN_FRAMEWORKS=(
+            'AE.bridgesupport',             'ATS.bridgesupport',                    'AVFoundation.bridgesupport',
+            'AudioToolbox.bridgesupport',   'CarbonSound.bridgesupport',            'CommonPanels.bridgesupport',
+            'CoreAudio.bridgesupport',      'CoreMIDI.bridgesupport',               'CoreText.bridgesupport',
+            'CoreVideo.bridgesupport',      'GameplayKit.bridgesupport',            'HIServices.bridgesupport',
+            'HIToolbox.bridgesupport',      'IOBluetooth.bridgesupport',            'IOKit.bridgesupport',
+            'Kerberos.bridgesupport',       'LangAnalysis.bridgesupport',           'MPSImage.bridgesupport',
+            'ModelIO.bridgesupport',        'NavigationServices.bridgesupport',     'OSServices.bridgesupport',
+            'Python.bridgesupport',         'QD.bridgesupport',                     'SceneKit.bridgesupport',
+            'Security.bridgesupport',       'SpeechSynthesis.bridgesupport',        'SpriteKit.bridgesupport',
+            'System.bridgesupport',         'TWAIN.bridgesupport',                  'Tk.bridgesupport',
+            'Tcl.bridgesupport',            'Vision.bridgesupport',		    'MPSRayIntersector.bridgesupport',
+            )
 
-            fn = os.path.join(framework_dir, dn, 'Frameworks')
-            if os.path.exists(fn):
-                for item in self.iter_framework_dir(fn):
-                    yield item
-
-    def iter_system_bridgesupport_files(self):
-        for item in self.iter_framework_dir('/System/Library/Frameworks'):
-            yield item
-
-    def test_system_bridgesupport(self):
-        with filterWarnings("ignore", RuntimeWarning):
-            # Check that all system bridgesupport files can be processed correctly
-            for fn in self.iter_system_bridgesupport_files():
-                with open(fn, 'r') as fp:
-                    xmldata = fp.read()
+    for fn in iter_system_bridgesupport_files():
+        _test_name = 'test_' + fn.replace('/System/Library/Frameworks', '').replace('/', '_').replace('.bridgesupport', '').replace('.framework', '')
+        def test_func(self, fn=fn):
+            with open(fn, 'rb') as fp:
+                xmldata = fp.read()
 
                 self.assert_valid_bridgesupport(os.path.basename(fn).split('.')[0], xmldata)
 
+        test_func.__name__ = _test_name
+        test_func.__doc__ = "System bridgesupport %r"%(fn,)
+
+        if contains_any(fn, BROKEN_FRAMEWORKS) and any(os_release().startswith(version) for version in ('10.13', '10.14')):
+            locals()[_test_name] = expectedFailure(test_func)
+        else:
+            locals()[_test_name] = test_func
+        del test_func
+        del _test_name
+
+
     def test_xml_structure_variants(self):
         # Run 'verify_xml_structure' for all cpu variant
         # (big/little endian,  32- en 64-bit)
@@ -1165,7 +1183,19 @@ class TestBridgeSupportParser (TestCase):
             self.assertEqual(set(meta.keys()) - {"arguments", "retval", "variadic", "suggestion", "classmethod" }, set())
 
     def assert_valid_bridgesupport(self, framework_name, xmldata):
-        prs = bridgesupport._BridgeSupportParser(xmldata, framework_name)
+        try:
+            prs = bridgesupport._BridgeSupportParser(xmldata, framework_name)
+        except objc.internal_error as exc:
+            if 'PyObjCRT_SkipTypeSpec: Unhandled type' in str(exc):
+                self.fail("Bad type encoding in metadata (bad type)")
+            elif 'Invalid array definition' in str(exc):
+                self.fail("Bad type encoding in metadata (bad array)")
+            elif 'Invalid union definition in type signature' in str(exc):
+                self.fail("Bad type encoding in metadata (bad union)")
+            elif 'Invalid struct definition in type signature' in str(exc):
+                self.fail("Bad type encoding in metadata (bad struct)")
+            else:
+                raise
 
         for item in prs.cftypes:
             if len(item) == 3:
@@ -1286,8 +1316,8 @@ class TestParseBridgeSupport (TestCase):
         class InlineTab (object): pass
         def loadConstant(name, typestr, magic):
             self.assertIsInstance(name, str)
-            self.assertIsInstance(typestr, bytes)
-            self.assertEqual(len(objc.splitSignature(typestr)), 1)
+            self.assertIsInstance(typestr, str)
+            self.assertEqual(len(objc.splitSignature(typestr.encode('ascii'))), 1)
             self.assertIsInstance(magic, bool)
 
             if 'raise' in name:
@@ -1594,12 +1624,15 @@ class TestInitFrameworkWrapper (TestCase):
             raise_exception = objc.internal_error
             update_globals = None
 
-            with filterWarnings("error", RuntimeWarning):
-                self.assertRaises(RuntimeWarning, bridgesupport._parseBridgeSupport, '', {}, 'TestFramework')
-
+            with warnings.catch_warnings(record=True) as w:
+                warnings.simplefilter("always")
+                bridgesupport._parseBridgeSupport( '', {}, 'TestFramework')
+            self.assertTrue(len(w) == 1)
+            self.assertEqual(w[0].category, RuntimeWarning)
             calls = []
 
-            with filterWarnings("ignore", RuntimeWarning):
+            with warnings.catch_warnings(record=True):
+                warnings.simplefilter("always")
                 g = {}
                 update_globals = {'foo': 42, 'bar': 33,  'protocols': MockModule() }
                 bridgesupport._parseBridgeSupport('', g, 'TestFramework')
@@ -2084,10 +2117,15 @@ class TestMisc (TestCase):
     def test_struct_alias(self):
         tp1 = objc.createStructType('TestStruct1', b'{TestStruct1="f1"d"f2"d}', None)
 
-        with filterWarnings("error", DeprecationWarning):
-            self.assertRaises(DeprecationWarning, objc.registerStructAlias, b'{TestStruct2=dd}', tp1)
+        with warnings.catch_warnings(record=True) as w:
+            warnings.simplefilter("always")
+            objc.registerStructAlias(b'{TestStruct2=dd}', tp1)
 
-        with filterWarnings("ignore", DeprecationWarning):
+        self.assertTrue(len(w) == 1)
+        self.assertEqual(w[0].category, DeprecationWarning)
+
+        with warnings.catch_warnings(record=True):
+            warnings.simplefilter("ignore")
             tp2 = objc.registerStructAlias(b'{TestStruct2=dd}', tp1)
             self.assertIs(tp1, tp2)
 
