@@ -12,7 +12,6 @@ TODO:
 from __future__ import unicode_literals
 import objc
 from PyObjCTools.TestSupport import *
-import warnings
 import array
 import sys
 
@@ -1168,6 +1167,26 @@ class TestVariableLengthValue (TestCase):
         self.assertEqual(v[5], 5)
         self.assertEqual(v[8], 8)
 
+        data = v.as_buffer(4)
+        if sys.version_info[0] == 2:
+            self.assertEqual(data[0], b'\x00')
+        else:
+            self.assertEqual(data[0], 0)
+        v[0] = 0x0f0f0f0f
+        if sys.version_info[0] == 2:
+            self.assertEqual(data[0], b'\x0f')
+            data[0] = b'\x00'
+
+        else:
+            self.assertEqual(data[0], 0x0f)
+            data[0] = 0
+
+        if sys.byteorder == 'little':
+            self.assertEqual(v[0], 0x0f0f0f00)
+
+        else:
+            self.assertEqual(v[0], 0x000f0f0f)
+
     def testInput(self):
         o = OC_MetaDataTest.alloc().init()
 
