@@ -6,6 +6,8 @@ from PyObjCTest import copying
 
 import objc, sys
 import functools
+import warnings
+import objc
 from PyObjCTest.fnd import NSObject, NSAutoreleasePool
 
 rct = structargs.StructArgClass.someRect.__metadata__()['retval']['type']
@@ -58,40 +60,35 @@ class TestRegressions(TestCase):
 
 
     def testDeallocUninit(self):
-        import objc
+        with warnings.catch_warnings():
+            warnings.filterwarnings('ignore',
+                category=objc.UninitializedDeallocWarning)
 
-        import warnings
-        warnings.filterwarnings('ignore',
-            category=objc.UninitializedDeallocWarning)
-
-        try:
             for clsName in [ 'NSURL', 'NSObject', 'NSArray' ]:
                 d = objc.lookUpClass(clsName).alloc()
                 del d
 
-        finally:
-            del warnings.filters[0]
 
         # Check that we generate a warning for unitialized objects that
         # get deallocated
         import sys
+        import gc
         if sys.version_info[0] == 2:
             from StringIO import StringIO
         else:
             from io import StringIO
-        warnings.filterwarnings('always',
-            category=objc.UninitializedDeallocWarning)
-        sys.stderr = buf = StringIO()
-        try:
+
+        with warnings.catch_warnings(record=True) as w:
             d = NSObject.alloc()
             del d
 
-        finally:
-            del warnings.filters[0]
-            sys.stderr = sys.__stderr__
+            # Expliclty for garbage collection, without this
+            # there are sporadic test failures when using
+            # coverage.py
+            gc.collect()
 
-        # A warning is three lines: location info, source code, empty line
-        self.assertEqual(len(buf.getvalue().split('\n')), 3)
+        self.assertTrue(len(w) == 1)
+        self.assertEqual(w[0].category, objc.UninitializedDeallocWarning)
 
     def testOneWayMethods(self):
         # This one should be in test_methods*.py
@@ -127,7 +124,26 @@ class TestRegressions(TestCase):
         self.assertRaises(TypeError, ClsIsNone.f, None)
 
 
-    def testStructArgs (self):
+    def testBufferArg(self):
+        data = objc.lookUpClass('NSData')
+
+        o = structargs.StructArgClass.alloc().init()
+
+        s = (b"foobar", 4)
+
+        d = o.dataFromBuffer_(s)
+        self.assertIsInstance(d, data)
+        self.assertEqual(d.length(), 4)
+        self.assertEqual(d.bytes(), b'foob')
+
+        s = (bytearray(b"FOOBAR"), 3)
+
+        d = o.dataFromBuffer_(s)
+        self.assertIsInstance(d, data)
+        self.assertEqual(d.length(), 3)
+        self.assertEqual(d.bytes(), b'FOO')
+
+    def testStructArgs(self):
         # Like AppKit.test.test_nsimage.TestNSImage.test_compositePoint
         # unlike that this one doesn't crash on darwin/x86, makeing it less
         # likely that libffi is at fault
@@ -242,20 +258,13 @@ class TestInitMemoryLeak (TestCase):
 
     def testInitFailureLeaks(self):
         NSData = objc.lookUpClass('NSData')
-        import warnings
-        warnings.filterwarnings('error',
-            category=objc.UninitializedDeallocWarning)
 
-        try:
-            try:
-                v = NSData.alloc().initWithContentsOfFile_("/etc/no-such-file.txt")
-            finally:
-                del warnings.filters[0]
+        with warnings.catch_warnings(record=True) as w:
+            v = NSData.alloc().initWithContentsOfFile_("/etc/no-such-file.txt")
+            self.assertFalse(v is not None)
+            del v
 
-        except objc.UninitializedDeallocWarning:
-            self.fail("Unexpected raising of UninitializedDeallocWarning")
-
-        self.assertFalse(v is not None)
+        self.assertEqual(w, [])
 
     def testExplicitGetItem(self):
         v = OCTestRegrWithGetItem.alloc().init()
@@ -267,29 +276,63 @@ class TestInitMemoryLeak (TestCase):
         self.assertEqual(v.objectForKey_("foo"), "ofk2: foo")
         self.assertEqual(v["foo"], "gi: foo")
 
-    def testExceptionOnWrapper(self):
-        # NOTE: I'm not to happy about the type error and will
-        #       likely change PyObjC. The test primarily checks
-        #       that this code doesn't crash the interpreter.
+    def testAllowDecorators(self):
 
         def decorator(function):
             @functools.wraps(function)
             def wrapper(*args, **kwds):
-                return function(*args, **kwds)
+                return function(*args, **kwds) * 2
 
             return wrapper
 
-        try:
-            class OCTestRegrWithWrapped (NSObject):
-                @decorator
-                def someMethod(self):
-                    pass
+        class OCTestRegrWithWrapped (NSObject):
+            @decorator
+            def someMethod(self):
+                return 42
 
-        except TypeError:
+        o = OCTestRegrWithWrapped.alloc().init()
+        v = o.someMethod()
+        self.assertEqual(v, 84)
+
+
+    def testDefaultSignatures(self):
+        def meth(): pass
+        s = objc.selector(meth)
+        self.assertEqual(s.selector, b'meth')
+        self.assertEqual(s.signature, b'v@:')
+
+        def meth__(): pass
+        s = objc.selector(meth__)
+        self.assertEqual(s.selector, b'meth::')
+        self.assertEqual(s.signature, b'v@:@@')
+
+        def meth(): return 1
+        s = objc.selector(meth)
+        self.assertEqual(s.selector, b'meth')
+        self.assertEqual(s.signature, b'@@:')
+
+        def meth__(): return 1
+        s = objc.selector(meth__)
+        self.assertEqual(s.selector, b'meth::')
+        self.assertEqual(s.signature, b'@@:@@')
+
+
+class TestNSInvocationBug (TestCase):
+    def testNSInvocationOveride(self):
+        class HelperNSInvocationTest (NSObject):
+            def forwardInvocation_(self, inv):
+                pass
+
+class TestDocTestProblem (TestCase):
+    def test_doctest(self):
+        import doctest
+
+        class DocTestHelper (NSObject):
             pass
 
-        else:
-            self.fail("TypeError not raised")
+
+        f = doctest.DocTestFinder(verbose=False)
+        f.find(DocTestHelper)
 
 if __name__ == '__main__':
     main()
