@@ -34,13 +34,16 @@ class TestStructs (TestCase):
         self.assertHasAttr(o, 'c')
         self.assertHasAttr(o, 'd')
 
-        self.assertEqual(len(o), 4)
+        with warnings.catch_warnings(record=True):
+            self.assertEqual(len(o), 4)
 
         self.assertHasAttr(objc.ivar, 'FooStruct')
         v = objc.ivar.FooStruct()
         self.assertIsInstance(v, objc.ivar)
         self.assertEqual(v.__typestr__, tp.__typestr__)
 
+        self.assertIsSubclass(tp, objc._structwrapper)
+
 
     def testNamedTupleAPI(self):
         Point = objc.createStructType("OCPoint", b"{_OCPoint=dd}", ["x", "y"])
@@ -118,52 +121,54 @@ class TestStructs (TestCase):
         self.assertHasAttr(o, 'h')
 
     def testEmbeddedFields(self):
-        tp = objc.createStructType("BarStruct", b'{FooStruct="first"i"second"i}', None)
+        with pyobjc_options(structs_indexable=False):
+            tp = objc.createStructType("BarStruct", b'{FooStruct="first"i"second"i}', None)
 
-        v = OC_StructTest.createWithFirst_andSecond_(1, 2)
-        self.assertIsInstance(v, tp)
+            v = OC_StructTest.createWithFirst_andSecond_(1, 2)
+            self.assertIsInstance(v, tp)
 
-        x = OC_StructTest.sumFields_(v)
-        self.assertEqual(x, v.first + v.second)
-        self.assertEqual(v.first, 1)
-        self.assertEqual(v.second, 2)
+            x = OC_StructTest.sumFields_(v)
+            self.assertEqual(x, v.first + v.second)
+            self.assertEqual(v.first, 1)
+            self.assertEqual(v.second, 2)
 
-        self.assertHasAttr(objc.ivar, 'BarStruct')
-        v = objc.ivar.BarStruct()
-        self.assertEqual(v.__typestr__, b'{FooStruct=ii}')
+            self.assertHasAttr(objc.ivar, 'BarStruct')
+            v = objc.ivar.BarStruct()
+            self.assertEqual(v.__typestr__, b'{FooStruct=ii}')
 
     def testStructCallback(self):
         """
         Regression test for an issue reported on the PyObjC mailinglist.
         """
-        tp = objc.createStructType("FooStruct", b'{FooStruct="first"i"second"i}', None)
+        with pyobjc_options(structs_indexable=False):
+            tp = objc.createStructType("FooStruct", b'{FooStruct="first"i"second"i}', None)
 
-        StructArrayDelegate = objc.informal_protocol(
-            "StructArrayDelegate",
-            [
-                objc.selector(None, b"arrayOf4Structs:",
-                    signature=b"@@:[4{FooStruct=ii}]"),
-            ])
+            StructArrayDelegate = objc.informal_protocol(
+                "StructArrayDelegate",
+                [
+                    objc.selector(None, b"arrayOf4Structs:",
+                        signature=b"@@:[4{FooStruct=ii}]"),
+                ])
 
-        class OC_PyStruct (NSObject):
+            class OC_PyStruct (NSObject):
 
-            def arrayOf4Structs_(self, value):
-                return value
+                def arrayOf4Structs_(self, value):
+                    return value
 
-        self.assertEqual(OC_PyStruct.arrayOf4Structs_.signature, b"@@:[4{FooStruct=" + objc._C_INT + objc._C_INT + b"}]")
+            self.assertEqual(OC_PyStruct.arrayOf4Structs_.signature, b"@@:[4{FooStruct=" + objc._C_INT + objc._C_INT + b"}]")
 
-        o = OC_PyStruct.alloc().init()
-        v = OC_StructTest.callArrayOf4Structs_(o)
-        self.assertEqual(len(v), 4)
-        for i in range(3):
-            self.assertIsInstance(v[i], tp)
+            o = OC_PyStruct.alloc().init()
+            v = OC_StructTest.callArrayOf4Structs_(o)
+            self.assertEqual(len(v), 4)
+            for i in range(3):
+                self.assertIsInstance(v[i], tp)
 
-        self.assertEqual(v[0], tp(1, 2))
-        self.assertEqual(v[1], tp(3, 4))
-        self.assertEqual(v[2], tp(5, 6))
-        self.assertEqual(v[3], tp(7, 8))
+            self.assertEqual(v[0], tp(1, 2))
+            self.assertEqual(v[1], tp(3, 4))
+            self.assertEqual(v[2], tp(5, 6))
+            self.assertEqual(v[3], tp(7, 8))
 
-    def testSTructSize(self):
+    def testStructSize(self):
         tp0 = objc.createStructType("FooStruct", b'{FooStruct=}', None)
         tp1 = objc.createStructType("FooStruct", b'{FooStruct="first"i}', None)
         tp2 = objc.createStructType("FooStruct", b'{FooStruct="first"i"second"i}', None)
@@ -171,5 +176,80 @@ class TestStructs (TestCase):
         self.assertEqual(sys.getsizeof(tp0()) + 1 * PTR_SIZE, sys.getsizeof(tp1()))
         self.assertEqual(sys.getsizeof(tp0()) + 2 * PTR_SIZE, sys.getsizeof(tp2()))
 
+    def testStructNotWritable(self):
+        tp0 = objc.createStructType("FooStruct", b'{FooStruct="first"i"second"i}', None)
+
+        with pyobjc_options(structs_indexable=False, structs_writable=False):
+            v = tp0(1,2)
+
+            with self.assertRaises(TypeError):
+                v.first = 42
+
+        with pyobjc_options(structs_indexable=True, structs_writable=False):
+            v = tp0(1,2)
+
+            with self.assertRaises(TypeError):
+                v.first = 42
+
+            with self.assertRaises(TypeError):
+                v[0] = 42
+
+
+    def testStructNotSequence(self):
+        tp0 = objc.createStructType("FooStruct", b'{FooStruct="first"i"second"i}', None)
+
+        with pyobjc_options(structs_indexable=False, structs_writable=True):
+            v = tp0(1, 2)
+
+            with self.assertRaises(TypeError):
+                v[0]
+
+            with self.assertRaises(TypeError):
+                v[0] = 4
+
+            with self.assertRaises(TypeError):
+                len(v)
+
+            self.assertFalse(v == (1, 2))
+
+            self.assertTrue(tp0(1,2) == tp0(1,2))
+            self.assertFalse(tp0(1,2) == tp0(1,3))
+
+            self.assertTrue(tp0(1,2) != tp0(1,3))
+            self.assertFalse(tp0(1,2) != tp0(1,2))
+
+            self.assertTrue(tp0(1,2) <= tp0(1,2))
+            self.assertTrue(tp0(1,2) <= tp0(1,3))
+            self.assertFalse(tp0(1,2) <= tp0(0,2))
+            self.assertFalse(tp0(1,2) <= tp0(1,1))
+
+            self.assertTrue(tp0(1,2) >= tp0(1,2))
+            self.assertTrue(tp0(1,2) >= tp0(1,1))
+            self.assertFalse(tp0(1,2) >= tp0(2,2))
+            self.assertFalse(tp0(1,2) >= tp0(1,3))
+
+            self.assertTrue(tp0(1,2) < tp0(1,3))
+            self.assertFalse(tp0(1,2) < tp0(1,2))
+            self.assertFalse(tp0(1,2) < tp0(1,1))
+
+            self.assertTrue(tp0(1,2) > tp0(1,1))
+            self.assertFalse(tp0(1,2) > tp0(1,2))
+            self.assertFalse(tp0(1,2) > tp0(1,3))
+
+
+    def testStructConstruction(self):
+        with pyobjc_options(structs_indexable=False):
+            tp0 = objc.createStructType("FooStruct", b'{FooStruct="first"i"second"i}', None)
+
+            v = tp0()
+            self.assertEqual(v.first, 0)
+            self.assertEqual(v.second, 0)
+
+            v = tp0(2, 3)
+            self.assertEqual(v.first, 2)
+            self.assertEqual(v.second, 3)
+
+            self.assertRaises(TypeError, tp0, 4, 5, 6)
+
 if __name__ == "__main__":
     main()
